@@ -242,6 +242,7 @@ struct ipmi_rs {
 #define IPMI_NETFN_TRANSPORT	0xc
 #define IPMI_NETFN_PICMG		0x2C
 #define IPMI_NETFN_DCGRP		0x2C
+#define IPMI_NETFN_OEM		0x2E
 #define IPMI_NETFN_ISOL			0x34
 #define IPMI_NETFN_TSOL			0x30
 
@@ -265,6 +266,7 @@ typedef enum IPMI_OEM {
      IPMI_OEM_DELL       = 674,
      IPMI_OEM_LMC        = 2168,
      IPMI_OEM_RADISYS    = 4337,
+     IPMI_OEM_BROADCOM   = 4413,
      IPMI_OEM_MAGNUM     = 5593,
      IPMI_OEM_TYAN       = 6653,
      IPMI_OEM_NEWISYS    = 9237,
