@@ -171,7 +171,7 @@ ipmi_get_channel_auth_cap(struct ipmi_intf * intf,
 			   auth_cap.oem_aux_data);
 	}
 
-    return 0;
+	return 0;
 }
 
 
@@ -438,7 +438,7 @@ ipmi_get_user_access(struct ipmi_intf * intf, uint8_t channel, uint8_t userid)
 static int
 ipmi_set_user_access(struct ipmi_intf * intf, int argc, char ** argv)
 {
-	uint8_t channel, userid;
+	uint8_t channel, privilege_limit, userid;
 	struct ipmi_rs * rsp;
 	struct ipmi_rq req;
 	uint8_t rqdata[2];
@@ -446,13 +446,19 @@ ipmi_set_user_access(struct ipmi_intf * intf, int argc, char ** argv)
 	struct set_user_access_data set_access;
 	int i;
 
-        if ((argc < 3) || (strncmp(argv[0], "help", 4) == 0)) {
+	if ((argc < 3) || (strncmp(argv[0], "help", 4) == 0)) {
 		printf_channel_usage();
-                return 0;
-        }
+		return 0;
+	}
 
-	channel = (uint8_t)strtol(argv[0], NULL, 0);
-	userid = (uint8_t)strtol(argv[1], NULL, 0);
+	if (str2uchar(argv[0], &channel) != 0) {
+		lprintf(LOG_ERR, "Numeric value expected, but '%s' given.", argv[0]);
+		return (-1);
+	}
+	if (str2uchar(argv[1], &userid) != 0) {
+		lprintf(LOG_ERR, "Numeric value expected, but '%s' given.", argv[1]);
+		return (-1);
+	}
 
 	memset(&req, 0, sizeof(req));
 	req.msg.netfn = IPMI_NETFN_APP;
@@ -500,7 +506,11 @@ ipmi_set_user_access(struct ipmi_intf * intf, int argc, char ** argv)
 			set_access.ipmi_messaging = strncmp (argv[i]+5, "off", 3);
 		}
 		else if (strncmp(argv[i], "privilege=", 10) == 0) {
-			set_access.privilege_limit = strtol (argv[i]+10, NULL, 0);
+			if (str2uchar(argv[i]+10, &privilege_limit) != 0) {
+				lprintf(LOG_ERR, "Numeric value expected, but '%s' given.", argv[i]+10);
+				return (-1);
+			}
+			set_access.privilege_limit = privilege_limit;
 		}
 		else {
 			printf ("Invalid option: %s\n", argv [i]);
@@ -593,9 +603,9 @@ ipmi_get_channel_cipher_suites(struct ipmi_intf * intf,
 	if (rsp->data_len >= 1)
 		channel = rsp->data[0];
 		
-   while ((rsp->data_len > 1) && (rsp->data_len == 17) && (list_index < 0x3F))
+	while ((rsp->data_len > 1) && (rsp->data_len == 17) && (list_index < 0x3F))
 	{
-   	//
+		//
 		// We got back cipher suite data -- store it.
 		//printf("copying data to offset %d\n", offset);
 		//printbuf(rsp->data + 1, rsp->data_len - 1, "this is the data");
@@ -620,15 +630,15 @@ ipmi_get_channel_cipher_suites(struct ipmi_intf * intf,
 		}
 	}
 
-   /* Copy last chunk */
-   if(rsp->data_len > 1)
-   {
-      //
-	   // We got back cipher suite data -- store it.
-	   //printf("copying data to offset %d\n", offset);
-	   //printbuf(rsp->data + 1, rsp->data_len - 1, "this is the data");
-	   memcpy(cipher_suite_data + offset, rsp->data + 1, rsp->data_len - 1);
-	   offset += rsp->data_len - 1;
+	/* Copy last chunk */
+	if(rsp->data_len > 1)
+	{
+		//
+		// We got back cipher suite data -- store it.
+		//printf("copying data to offset %d\n", offset);
+		//printbuf(rsp->data + 1, rsp->data_len - 1, "this is the data");
+		memcpy(cipher_suite_data + offset, rsp->data + 1, rsp->data_len - 1);
+		offset += rsp->data_len - 1;
 	}
 
 	//
@@ -752,7 +762,7 @@ ipmi_get_channel_medium(struct ipmi_intf * intf, uint8_t channel)
 	rsp = intf->sendrecv(intf, &req);
 	if (rsp == NULL) {
 		lprintf(LOG_ERR, "Get Channel Info command failed");
-		return -1;
+		return 0;
 	}
 	if (rsp->ccode > 0) {
 		if (rsp->ccode == 0xcc)
@@ -799,6 +809,7 @@ int
 ipmi_channel_main(struct ipmi_intf * intf, int argc, char ** argv)
 {
 	int retval = 0;
+	uint8_t channel, priv = 0;
 
 	if ((argc == 0) || (strncmp(argv[0], "help", 4) == 0))
 	{
@@ -806,22 +817,38 @@ ipmi_channel_main(struct ipmi_intf * intf, int argc, char ** argv)
 	}
 	else if (strncmp(argv[0], "authcap", 7) == 0)
 	{
-		if (argc != 3)
+		if (argc != 3) {
 			printf_channel_usage();
-		else
-			retval = ipmi_get_channel_auth_cap(intf,
-                                               (uint8_t)strtol(argv[1], NULL, 0),
-                                               (uint8_t)strtol(argv[2], NULL, 0));
+			return (-1);
+		} else {
+			if (str2uchar(argv[1], &channel) != 0) {
+				lprintf(LOG_ERR, "Numeric value expected, but '%s' given.", argv[1]);
+				return (-1);
+			}
+			if (str2uchar(argv[2], &priv) != 0) {
+				lprintf(LOG_ERR, "Numeric value expected, but '%s' given.", argv[2]);
+				return (-1);
+			}
+			retval = ipmi_get_channel_auth_cap(intf, channel, priv);
+		}
 	}
 	else if (strncmp(argv[0], "getaccess", 10) == 0)
 	{
 		if ((argc < 2) || (argc > 3))
 			printf_channel_usage();
 		else {
-			uint8_t ch = (uint8_t)strtol(argv[1], NULL, 0);
+			uint8_t ch = 0;
 			uint8_t id = 0;
-			if (argc == 3)
-				id = (uint8_t)strtol(argv[2], NULL, 0);
+			if (str2uchar(argv[1], &ch) != 0) {
+				lprintf(LOG_ERR, "Numeric value expected, but '%s' given.", argv[1]);
+				return (-1);
+			}
+			if (argc == 3) {
+				if (str2uchar(argv[2], &id) != 0) {
+					lprintf(LOG_ERR, "Numeric value expected, but '%s' given.", argv[2]);
+					return (-1);
+				}
+			}
 			retval = ipmi_get_user_access(intf, ch, id);
 		}
 	}
@@ -835,8 +862,12 @@ ipmi_channel_main(struct ipmi_intf * intf, int argc, char ** argv)
 			printf_channel_usage();
 		else {
 			uint8_t ch = 0xe;
-			if (argc == 2)
-				ch = (uint8_t)strtol(argv[1], NULL, 0);
+			if (argc == 2) {
+				if (str2uchar(argv[1], &ch) != 0) {
+					lprintf(LOG_ERR, "Numeric value expected, but '%s' given.", argv[1]);
+					return (-1);
+				}
+			}
 			retval = ipmi_get_channel_info(intf, ch);
 		}
 	}
@@ -850,8 +881,12 @@ ipmi_channel_main(struct ipmi_intf * intf, int argc, char ** argv)
 		else
 		{
 			uint8_t ch = 0xe;
-			if (argc == 3)
-				ch = (uint8_t)strtol(argv[2], NULL, 0);
+			if (argc == 3) {
+				if (str2uchar(argv[2], &ch) != 0) {
+					lprintf(LOG_ERR, "Numeric value expected, but '%s' given.", argv[2]);
+					return (-1);
+				}
+			}
 			retval = ipmi_get_channel_cipher_suites(intf,
 								argv[1], // ipmi | sol
 								ch);
