@@ -608,7 +608,7 @@ ipmi_isol_red_pill(struct ipmi_intf * intf)
 			 */
 			if (FD_ISSET(0, &read_fds))
 	 		{
-				bzero(buffer, sizeof(buffer));
+				memset(buffer, 0, buffer_size);
 				numRead = read(fileno(stdin),
 							   buffer,
 							   buffer_size);
