@@ -74,9 +74,9 @@
 #endif
 
 #ifdef ENABLE_ALL_OPTIONS
-# define OPTION_STRING	"I:hVvcgsEKYao:H:d:P:f:U:p:C:L:A:t:T:m:z:S:l:b:B:e:k:y:O:R:N:"
+# define OPTION_STRING	"I:hVvcgsEKYao:H:d:P:f:U:p:C:L:A:t:T:m:z:S:l:b:B:e:k:y:O:R:N:D:"
 #else
-# define OPTION_STRING	"I:hVvcH:f:U:p:d:S:"
+# define OPTION_STRING	"I:hVvcH:f:U:p:d:S:D:"
 #endif
 
 extern int verbose;
@@ -84,13 +84,6 @@ extern int csv_output;
 extern const struct valstr ipmi_privlvl_vals[];
 extern const struct valstr ipmi_authtype_session_vals[];
 
-/* defined in ipmishell.c */
-#ifdef HAVE_READLINE
-extern int ipmi_shell_main(struct ipmi_intf * intf, int argc, char ** argv);
-#endif
-extern int ipmi_set_main(struct ipmi_intf * intf, int argc, char ** argv);
-extern int ipmi_exec_main(struct ipmi_intf * intf, int argc, char ** argv);
-
 static struct ipmi_intf * ipmi_main_intf = NULL;
 
 /* ipmi_password_file_read  -  Open file and read password from it
@@ -108,23 +101,26 @@ ipmi_password_file_read(char * filename)
 	char * pass = NULL;
 	int l;
 
-	pass = malloc(16);
+	pass = malloc(21);
 	if (pass == NULL) {
 		lprintf(LOG_ERR, "ipmitool: malloc failure");
 		return NULL;
 	}
 
+	memset(pass, 0, 21);
 	fp = ipmi_open_file_read((const char *)filename);
 	if (fp == NULL) {
 		lprintf(LOG_ERR, "Unable to open password file %s",
 				filename);
+		free(pass);
 		return NULL;
 	}
 
 	/* read in id */
-	if (fgets(pass, 16, fp) == NULL) {
+	if (fgets(pass, 21, fp) == NULL) {
 		lprintf(LOG_ERR, "Unable to read password from file %s",
 				filename);
+		free(pass);
 		fclose(fp);
 		return NULL;
 	}
@@ -229,6 +225,8 @@ ipmi_option_usage(const char * progname, struct ipmi_cmd * cmdlist, struct ipmi_
 	lprintf(LOG_NOTICE, "       -f file        Read remote session password from file");
 	lprintf(LOG_NOTICE, "       -z size        Change Size of Communication Channel (OEM)");
 	lprintf(LOG_NOTICE, "       -S sdr         Use local file for remote SDR cache");
+	lprintf(LOG_NOTICE, "       -D tty:b[:s]   Specify the serial device, baud rate to use");
+	lprintf(LOG_NOTICE, "                      and, optionally, specify that interface is the system one");
 #ifdef ENABLE_ALL_OPTIONS
 	lprintf(LOG_NOTICE, "       -a             Prompt for remote password");
 	lprintf(LOG_NOTICE, "       -Y             Prompt for the Kg key for IPMIv2 authentication");
@@ -245,7 +243,6 @@ ipmi_option_usage(const char * progname, struct ipmi_cmd * cmdlist, struct ipmi_
 	lprintf(LOG_NOTICE, "       -m address     Set local IPMB address");
 	lprintf(LOG_NOTICE, "       -b channel     Set destination channel for bridged request");
 	lprintf(LOG_NOTICE, "       -t address     Bridge request to remote target address");
-	lprintf(LOG_NOTICE, "       -M address     Set transit local address for bridge request(dual bridge)");
 	lprintf(LOG_NOTICE, "       -B channel     Set transit channel for bridged request (dual bridge)");
 	lprintf(LOG_NOTICE, "       -T address     Set transit address for bridge request (dual bridge)");
 	lprintf(LOG_NOTICE, "       -l lun         Set destination lun for raw commands");
@@ -320,6 +317,7 @@ ipmi_parse_hex(const char *str)
 		if (!isxdigit(*p)) {
 			lprintf(LOG_ERR, "Kg_hex is not hexadecimal number");
 			free(out);
+			out = NULL;
 			return NULL;
 		}
 
@@ -363,14 +361,15 @@ ipmi_main(int argc, char ** argv,
 	uint8_t transit_addr = 0;
 	uint8_t transit_channel = 0;
 	uint8_t target_lun     = 0;
-	uint8_t my_addr = 0x20;
+	uint8_t arg_addr = 0, addr;
 	uint16_t my_long_packet_size=0;
 	uint8_t my_long_packet_set=0;
 	uint8_t lookupbit = 0x10;	/* use name-only lookup by default */
 	int retry = 0;
 	uint32_t timeout = 0;
 	int authtype = -1;
-	char * tmp = NULL;
+	char * tmp_pass = NULL;
+	char * tmp_env = NULL;
 	char * hostname = NULL;
 	char * username = NULL;
 	char * password = NULL;
@@ -382,10 +381,11 @@ ipmi_main(int argc, char ** argv,
 	char * seloem   = NULL;
 	int port = 0;
 	int devnum = 0;
-	int cipher_suite_id = 1; /* See table 22-19 of the IPMIv2 spec */
+	int cipher_suite_id = 3; /* See table 22-19 of the IPMIv2 spec */
 	int argflag, i, found;
 	int rc = -1;
 	char sol_escape_char = SOL_ESCAPE_CHARACTER_DEFAULT;
+	char * devfile  = NULL;
 
 	/* save program name */
 	progname = strrchr(argv[0], '/');
@@ -396,6 +396,10 @@ ipmi_main(int argc, char ** argv,
 	{
 		switch (argflag) {
 		case 'I':
+			if (intfname) {
+				free(intfname);
+				intfname = NULL;
+			}
 			intfname = strdup(optarg);
 			if (intfname == NULL) {
 				lprintf(LOG_ERR, "%s: malloc failure", progname);
@@ -473,6 +477,10 @@ ipmi_main(int argc, char ** argv,
 			csv_output = 1;
 			break;
 		case 'H':
+			if (hostname) {
+				free(hostname);
+				hostname = NULL;
+			}
 			hostname = strdup(optarg);
 			if (hostname == NULL) {
 				lprintf(LOG_ERR, "%s: malloc failure", progname);
@@ -480,8 +488,10 @@ ipmi_main(int argc, char ** argv,
 			}
 			break;
 		case 'f':
-			if (password)
+			if (password) {
 				free(password);
+				password = NULL;
+			}
 			password = ipmi_password_file_read(optarg);
 			if (password == NULL)
 				lprintf(LOG_ERR, "Unable to read password "
@@ -489,14 +499,17 @@ ipmi_main(int argc, char ** argv,
 			break;
 		case 'a':
 #ifdef HAVE_GETPASSPHRASE
-			tmp = getpassphrase("Password: ");
+			tmp_pass = getpassphrase("Password: ");
 #else
-			tmp = getpass("Password: ");
+			tmp_pass = getpass("Password: ");
 #endif
-			if (tmp != NULL) {
-				if (password)
+			if (tmp_pass != NULL) {
+				if (password) {
 					free(password);
-				password = strdup(tmp);
+					password = NULL;
+				}
+				password = strdup(tmp_pass);
+				tmp_pass = NULL;
 				if (password == NULL) {
 					lprintf(LOG_ERR, "%s: malloc failure", progname);
 					goto out_free;
@@ -504,6 +517,10 @@ ipmi_main(int argc, char ** argv,
 			}
 			break;
 		case 'k':
+			if (kgkey) {
+				free(kgkey);
+				kgkey = NULL;
+			}
 			kgkey = strdup(optarg);
 			if (kgkey == NULL) {
 				lprintf(LOG_ERR, "%s: malloc failure", progname);
@@ -511,10 +528,12 @@ ipmi_main(int argc, char ** argv,
 			}
 			break;
 		case 'K':
-			if ((tmp = getenv("IPMI_KGKEY"))) {
-				if (kgkey)
+			if ((tmp_env = getenv("IPMI_KGKEY"))) {
+				if (kgkey) {
 					free(kgkey);
-				kgkey = strdup(tmp);
+					kgkey = NULL;
+				}
+				kgkey = strdup(tmp_env);
 				if (kgkey == NULL) {
 					lprintf(LOG_ERR, "%s: malloc failure", progname);
 					goto out_free;
@@ -524,6 +543,10 @@ ipmi_main(int argc, char ** argv,
 			}
 			break;
 		case 'y':
+			if (kgkey) {
+				free(kgkey);
+				kgkey = NULL;
+			}
 			kgkey = ipmi_parse_hex(optarg);
 			if (kgkey == NULL) {
 				goto out_free;
@@ -531,14 +554,17 @@ ipmi_main(int argc, char ** argv,
 			break;
 		case 'Y':
 #ifdef HAVE_GETPASSPHRASE
-			tmp = getpassphrase("Key: ");
+			tmp_pass = getpassphrase("Key: ");
 #else
-			tmp = getpass("Key: ");
+			tmp_pass = getpass("Key: ");
 #endif
-			if (tmp != NULL) {
-				if (kgkey)
+			if (tmp_pass != NULL) {
+				if (kgkey) {
 					free(kgkey);
-				kgkey = strdup(tmp);
+					kgkey = NULL;
+				}
+				kgkey = strdup(tmp_pass);
+				tmp_pass = NULL;
 				if (kgkey == NULL) {
 					lprintf(LOG_ERR, "%s: malloc failure", progname);
 					goto out_free;
@@ -546,6 +572,10 @@ ipmi_main(int argc, char ** argv,
 			}
 			break;
 		case 'U':
+			if (username) {
+				free(username);
+				username = NULL;
+			}
 			if (strlen(optarg) > 16) {
 				lprintf(LOG_ERR, "Username is too long (> 16 bytes)");
 				goto out_free;
@@ -557,14 +587,34 @@ ipmi_main(int argc, char ** argv,
 			}
 			break;
 		case 'S':
+			if (sdrcache) {
+				free(sdrcache);
+				sdrcache = NULL;
+			}
 			sdrcache = strdup(optarg);
 			if (sdrcache == NULL) {
 				lprintf(LOG_ERR, "%s: malloc failure", progname);
 				goto out_free;
 			}
 			break;
+		case 'D':
+			/* check for subsequent instance of -D */
+			if (devfile) {
+				/* free memory for previous string */
+				free(devfile);
+			}
+			devfile = strdup(optarg);
+			if (devfile == NULL) {
+				lprintf(LOG_ERR, "%s: malloc failure", progname);
+				goto out_free;
+			}
+			break;
 #ifdef ENABLE_ALL_OPTIONS
 		case 'o':
+			if (oemtype) {
+				free(oemtype);
+				oemtype = NULL;
+			}
 			oemtype = strdup(optarg);
 			if (oemtype == NULL) {
 				lprintf(LOG_ERR, "%s: malloc failure", progname);
@@ -579,15 +629,25 @@ ipmi_main(int argc, char ** argv,
 			break;
 		case 'g':
 			/* backwards compatible oem hack */
+			if (oemtype) {
+				free(oemtype);
+				oemtype = NULL;
+			}
 			oemtype = strdup("intelwv2");
 			break;
 		case 's':
 			/* backwards compatible oem hack */
+			if (oemtype) {
+				free(oemtype);
+				oemtype = NULL;
+			}
 			oemtype = strdup("supermicro");
 			break;
 		case 'P':
-			if (password)
+			if (password) {
 				free(password);
+				password = NULL;
+			}
 			password = strdup(optarg);
 			if (password == NULL) {
 				lprintf(LOG_ERR, "%s: malloc failure", progname);
@@ -599,19 +659,23 @@ ipmi_main(int argc, char ** argv,
 			memset(optarg, 'X', i);
 			break;
 		case 'E':
-			if ((tmp = getenv("IPMITOOL_PASSWORD"))) {
-				if (password)
+			if ((tmp_env = getenv("IPMITOOL_PASSWORD"))) {
+				if (password) {
 					free(password);
-				password = strdup(tmp);
+					password = NULL;
+				}
+				password = strdup(tmp_env);
 				if (password == NULL) {
 					lprintf(LOG_ERR, "%s: malloc failure", progname);
 					goto out_free;
 				}
 			}
-			else if ((tmp = getenv("IPMI_PASSWORD"))) {
-				if (password)
+			else if ((tmp_env = getenv("IPMI_PASSWORD"))) {
+				if (password) {
 					free(password);
-				password = strdup(tmp);
+					password = NULL;
+				}
+				password = strdup(tmp_env);
 				if (password == NULL) {
 					lprintf(LOG_ERR, "%s: malloc failure", progname);
 					goto out_free;
@@ -671,7 +735,7 @@ ipmi_main(int argc, char ** argv,
 			}
 			break;
 		case 'm':
-			if (str2uchar(optarg, &my_addr) != 0) {
+			if (str2uchar(optarg, &arg_addr) != 0) {
 				lprintf(LOG_ERR, "Invalid parameter given or out of range for '-m'.");
 				rc = -1;
 				goto out_free;
@@ -681,6 +745,10 @@ ipmi_main(int argc, char ** argv,
 			sol_escape_char = optarg[0];
 			break;
 		case 'O':
+			if (seloem) {
+				free(seloem);
+				seloem = NULL;
+			}
 			seloem = strdup(optarg);
 			if (seloem == NULL) {
 				lprintf(LOG_ERR, "%s: malloc failure", progname);
@@ -735,12 +803,13 @@ ipmi_main(int argc, char ** argv,
 	if (hostname != NULL && password == NULL &&
 			(authtype != IPMI_SESSION_AUTHTYPE_NONE || authtype < 0)) {
 #ifdef HAVE_GETPASSPHRASE
-		tmp = getpassphrase("Password: ");
+		tmp_pass = getpassphrase("Password: ");
 #else
-		tmp = getpass("Password: ");
+		tmp_pass = getpass("Password: ");
 #endif
-		if (tmp != NULL) {
-			password = strdup(tmp);
+		if (tmp_pass != NULL) {
+			password = strdup(tmp_pass);
+			tmp_pass = NULL;
 			if (password == NULL) {
 				lprintf(LOG_ERR, "%s: malloc failure", progname);
 				goto out_free;
@@ -818,96 +887,90 @@ ipmi_main(int argc, char ** argv,
 	ipmi_intf_session_set_sol_escape_char(ipmi_main_intf, sol_escape_char);
 	ipmi_intf_session_set_cipher_suite_id(ipmi_main_intf, cipher_suite_id);
 
-	/* setup destination lun if given */
-	ipmi_main_intf->target_lun = target_lun ;
-
-	/* setup destination channel if given */
-	ipmi_main_intf->target_channel = target_channel ;
-
 	ipmi_main_intf->devnum = devnum;
 
-	/* setup IPMB local and target address if given */
-	if (my_addr) {
-		ipmi_main_intf->my_addr = my_addr;
-	} else {
-		/* Check if PICMG extension is available to use the function 
-		 * GetDeviceLocator to retreive i2c address PICMG hack to set 
-		 * right IPMB address, If extension is not supported, should 
-		 * not give any problems
-		 *  PICMG Extension Version 2.0 (PICMG 3.0 Revision 1.0 ATCA) to
-		 *  PICMG Extension Version 2.3 (PICMG 3.0 Revision 3.0 ATCA)
-		 */
+	/* setup device file if given */
+	ipmi_main_intf->devfile = devfile;
 
-		/* First, check if PICMG extension is available and supported */
-		struct ipmi_rq req;
-		struct ipmi_rs *rsp;
-		char msg_data;
-		unsigned char version_accepted = 0;
+	/* Open the interface with the specified or default IPMB address */
+	ipmi_main_intf->my_addr = arg_addr ? arg_addr : IPMI_BMC_SLAVE_ADDR;
+	if (ipmi_main_intf->open != NULL)
+		ipmi_main_intf->open(ipmi_main_intf);
 
-		lprintf(LOG_INFO, "Running PICMG GetDeviceLocator" );
-		memset(&req, 0, sizeof(req));
-		req.msg.netfn = IPMI_NETFN_PICMG;
-		req.msg.cmd = PICMG_GET_PICMG_PROPERTIES_CMD;
-		msg_data    = 0x00;
-		req.msg.data = &msg_data;
-		req.msg.data_len = 1;
-		msg_data = 0;
-
-		rsp = ipmi_main_intf->sendrecv(ipmi_main_intf, &req);
-		if (rsp && !rsp->ccode) {
-			if ( (rsp->data[0] == 0) &&
-					((rsp->data[1] & 0x0F) == PICMG_ATCA_MAJOR_VERSION) )	{
-				version_accepted = 1;
-				lprintf(LOG_INFO, "Discovered PICMG Extension %d.%d",
-						(rsp->data[1] & 0x0f), (rsp->data[1] >> 4));
-			}
-		}
-		
-		if (version_accepted == 1) {
-			lprintf(LOG_DEBUG, "Running PICMG GetDeviceLocator");
-			memset(&req, 0, sizeof(req));
-			req.msg.netfn = IPMI_NETFN_PICMG;
-			req.msg.cmd = PICMG_GET_ADDRESS_INFO_CMD;
-			msg_data    = 0x00;
-			req.msg.data = &msg_data;
-			req.msg.data_len = 1;
-			msg_data = 0;
-
-			rsp = ipmi_main_intf->sendrecv(ipmi_main_intf, &req);
-			if (rsp && !rsp->ccode) {
-				ipmi_main_intf->my_addr = rsp->data[2];
-				ipmi_main_intf->target_addr = ipmi_main_intf->my_addr;
-				lprintf(LOG_INFO, "Discovered IPMB address = 0x%x",
-						ipmi_main_intf->my_addr);
-			}
-		} else {
-			lprintf(LOG_INFO,
-					"No PICMG Extenstion discovered, keeping IPMB address 0x20");
-		}
+	/*
+	 * Attempt picmg discovery of the actual interface address unless
+	 * the users specified an address.
+	 *	Address specification always overrides discovery
+	 */
+	if (picmg_discover(ipmi_main_intf) && !arg_addr) {
+		lprintf(LOG_DEBUG, "Running PICMG Get Address Info");
+		addr = ipmi_picmg_ipmb_address(ipmi_main_intf);
+		lprintf(LOG_INFO,  "Discovered IPMB-0 address 0x%x", addr);
 	}
 
-	if ( target_addr > 0  && (target_addr != my_addr) ) {
-		/* need to open the interface first */
-		if (ipmi_main_intf->open != NULL)
-			ipmi_main_intf->open(ipmi_main_intf);
+	/*
+	 * If we discovered the ipmb address and it is not the same as what we
+	 * used for open, Set the discovered IPMB address as my address if the
+	 * interface supports it.
+	 */
+	if (addr != 0 && addr != ipmi_main_intf->my_addr &&
+						ipmi_main_intf->set_my_addr) {
+		/*
+		 * Only set the interface address on interfaces which support
+		 * it
+		 */
+		(void) ipmi_main_intf->set_my_addr(ipmi_main_intf, addr);
+	}
 
-		ipmi_main_intf->target_addr = target_addr;
-
-		if (transit_addr > 0) {
-			ipmi_main_intf->transit_addr    = transit_addr;
-			ipmi_main_intf->transit_channel = transit_channel;
-		} else {
-			ipmi_main_intf->transit_addr = ipmi_main_intf->my_addr;
+	/* If bridging addresses are specified, handle them */
+	if (transit_addr > 0 || target_addr > 0) {
+		/* sanity check, transit makes no sense without a target */
+		if ((transit_addr != 0 || transit_channel != 0) &&
+			target_addr == 0) {
+			lprintf(LOG_ERR,
+				"Transit address/channel %#x/%#x ignored. "
+				"Target address must be specified!",
+				transit_addr, transit_channel);
+			goto out_free;
 		}
+		ipmi_main_intf->target_addr = target_addr;
+		ipmi_main_intf->target_lun = target_lun ;
+		ipmi_main_intf->target_channel = target_channel ;
+
+		ipmi_main_intf->transit_addr    = transit_addr;
+		ipmi_main_intf->transit_channel = transit_channel;
+
+
 		/* must be admin level to do this over lan */
 		ipmi_intf_session_set_privlvl(ipmi_main_intf, IPMI_SESSION_PRIV_ADMIN);
+		/* Get the ipmb address of the targeted entity */
+		ipmi_main_intf->target_ipmb_addr =
+					ipmi_picmg_ipmb_address(ipmi_main_intf);
+		lprintf(LOG_DEBUG, "Specified addressing     Target  %#x:%#x Transit %#x:%#x",
+					   ipmi_main_intf->target_addr,
+					   ipmi_main_intf->target_channel,
+					   ipmi_main_intf->transit_addr,
+					   ipmi_main_intf->transit_channel);
+		if (ipmi_main_intf->target_ipmb_addr) {
+			lprintf(LOG_INFO, "Discovered Target IPMB-0 address %#x",
+					   ipmi_main_intf->target_ipmb_addr);
+		}
 	}
 
+	lprintf(LOG_DEBUG, "Interface address: my_addr %#x "
+			   "transit %#x:%#x target %#x:%#x "
+			   "ipmb_target %#x\n",
+			ipmi_main_intf->my_addr,
+			ipmi_main_intf->transit_addr,
+			ipmi_main_intf->transit_channel,
+			ipmi_main_intf->target_addr,
+			ipmi_main_intf->target_channel,
+			ipmi_main_intf->target_ipmb_addr);
+
 	/* parse local SDR cache if given */
 	if (sdrcache != NULL) {
 		ipmi_sdr_list_cache_fromfile(ipmi_main_intf, sdrcache);
 	}
-
 	/* Parse SEL OEM file if given */
 	if (seloem != NULL) {
 		ipmi_sel_oem_init(seloem);
@@ -948,18 +1011,44 @@ ipmi_main(int argc, char ** argv,
 	out_free:
 	log_halt();
 
-	if (intfname != NULL)
+	if (intfname != NULL) {
 		free(intfname);
-	if (hostname != NULL)
+		intfname = NULL;
+	}
+	if (hostname != NULL) {
 		free(hostname);
-	if (username != NULL)
+		hostname = NULL;
+	}
+	if (username != NULL) {
 		free(username);
-	if (password != NULL)
+		username = NULL;
+	}
+	if (password != NULL) {
 		free(password);
-	if (oemtype != NULL)
+		password = NULL;
+	}
+	if (oemtype != NULL) {
 		free(oemtype);
-	if (seloem != NULL)
+		oemtype = NULL;
+	}
+	if (seloem != NULL) {
 		free(seloem);
+		seloem = NULL;
+	}
+	if (kgkey != NULL) {
+		free(kgkey);
+		kgkey = NULL;
+	}
+	if (sdrcache != NULL) {
+		free(sdrcache);
+		sdrcache = NULL;
+	}
+	if (devfile) {
+		free(devfile);
+		devfile = NULL;
+	}
 
 	return rc;
 }
+
+
