@@ -63,6 +63,10 @@ static struct ipmi_oem_handle ipmi_oem_list[] = {
 		desc:   "IBM OEM support",
 		setup:	ipmi_oem_ibm,
 	},
+	{
+		name:   "i82571spt",
+		desc:   "Intel 82571 MAC with integrated RMCP+ support in super pass-through mode",
+	},
 	{ 0 }
 };
 
@@ -112,9 +116,9 @@ ipmi_oem_setup(struct ipmi_intf * intf, char * oemtype)
 	struct ipmi_oem_handle * oem;
 	int rc = 0;
 
-	if (strncmp(oemtype, "help", 4) == 0 ||
-	    strncmp(oemtype, "list", 4) == 0 ||
-	    oemtype == NULL) {
+	if (oemtype == NULL ||
+			strncmp(oemtype, "help", 4) == 0 ||
+			strncmp(oemtype, "list", 4) == 0) {
 		ipmi_oem_print();
 		return -1;
 	}
