@@ -44,6 +44,9 @@
 
 #define IPMI_I2C_MASTER_MAX_SIZE	0x40 /* 64 bytes */
 
+static int is_valid_param(const char *input_param, uint8_t *uchr_ptr,
+		const char *label);
+
 /* ipmi_master_write_read  -  Perform I2C write/read transactions
  *
  * This function performs an I2C master write-read function through
@@ -133,6 +136,8 @@ int
 ipmi_rawspd_main(struct ipmi_intf * intf, int argc, char ** argv)
 {
 	struct ipmi_rs *rsp;
+	uint8_t msize = IPMI_I2C_MASTER_MAX_SIZE; /* allow to override default */
+	uint8_t channel = 0;
 	uint8_t i2cbus = 0;
 	uint8_t i2caddr = 0;
 	uint8_t spd_data[RAW_SPD_SIZE];
@@ -141,24 +146,37 @@ ipmi_rawspd_main(struct ipmi_intf * intf, int argc, char ** argv)
 	memset(spd_data, 0, RAW_SPD_SIZE);
 
 	if (argc < 2 || strncmp(argv[0], "help", 4) == 0) {
-		lprintf(LOG_NOTICE, "usage: spd <i2cbus> <i2caddr>");
+		lprintf(LOG_NOTICE, "usage: spd <i2cbus> <i2caddr> [channel] [maxread]");
 		return 0;
 	}
 
-	i2cbus  = (uint8_t)strtoul(argv[0], NULL, 0);
-	i2caddr = (uint8_t)strtoul(argv[1], NULL, 0);
+	if (is_valid_param(argv[0], &i2cbus, "i2cbus") != 0)
+		return (-1);
 
-	i2cbus = ((i2cbus & 7) << 1) | 1;
+	if (is_valid_param(argv[1], &i2caddr, "i2caddr") != 0)
+		return (-1);
 
-	for (i = 0; i < RAW_SPD_SIZE; i+= IPMI_I2C_MASTER_MAX_SIZE) {
+	if (argc >= 3) {
+		if (is_valid_param(argv[2], &channel, "channel") != 0)
+			return (-1);
+	}
+
+	if (argc >= 4) {
+		if (is_valid_param(argv[3], &msize, "maxread") != 0)
+			return (-1);
+	}
+
+	i2cbus = ((channel & 0xF) << 4) | ((i2cbus & 7) << 1) | 1;
+
+	for (i = 0; i < RAW_SPD_SIZE; i+= msize) {
 		rsp = ipmi_master_write_read(intf, i2cbus, i2caddr,
-					     (uint8_t *)&i, 1, IPMI_I2C_MASTER_MAX_SIZE);
+					     (uint8_t *)&i, 1, msize );
 		if (rsp == NULL) {
 			lprintf(LOG_ERR, "Unable to perform I2C Master Write-Read");
 			return -1;
 		}
 
-		memcpy(spd_data+i, rsp->data, IPMI_I2C_MASTER_MAX_SIZE);
+		memcpy(spd_data+i, rsp->data, msize);
 	}
 
 	ipmi_spd_print(spd_data, i);
@@ -167,7 +185,7 @@ ipmi_rawspd_main(struct ipmi_intf * intf, int argc, char ** argv)
 
 static void rawi2c_usage(void)
 {
-	lprintf(LOG_NOTICE, "usage: i2c [bus=public|# [chan=#]] <i2caddr> <read bytes> [write data]");
+	lprintf(LOG_NOTICE, "usage: i2c [bus=public|# [chan=#] <i2caddr> <read bytes> [write data]");
 	lprintf(LOG_NOTICE, "            bus=public is default");
 	lprintf(LOG_NOTICE, "            chan=0 is default, bus= must be specified to use chan=");
 }
@@ -213,8 +231,11 @@ ipmi_rawi2c_main(struct ipmi_intf * intf, int argc, char ** argv)
 		return -1;
 	}
 
-	i2caddr = (uint8_t)strtoul(argv[i++], NULL, 0);
-	rsize = (uint8_t)strtoul(argv[i++], NULL, 0);
+	if (is_valid_param(argv[i++], &i2caddr, "i2caddr") != 0)
+		return (-1);
+
+	if (is_valid_param(argv[i++], &rsize, "read size") != 0)
+		return (-1);
 
 	if (i2caddr == 0) {
 		lprintf(LOG_ERR, "Invalid I2C address 0");
@@ -224,8 +245,12 @@ ipmi_rawi2c_main(struct ipmi_intf * intf, int argc, char ** argv)
 
 	memset(wdata, 0, IPMI_I2C_MASTER_MAX_SIZE);
 	for (; i < argc; i++) {
-		uint8_t val = (uint8_t)strtol(argv[i], NULL, 0);
-		wdata[i-2] = val;
+		uint8_t val = 0;
+
+		if (is_valid_param(argv[i], &val, "parameter") != 0)
+			return (-1);
+
+		wdata[wsize] = val;
 		wsize++;
 	}
 
@@ -275,20 +300,36 @@ ipmi_rawi2c_main(struct ipmi_intf * intf, int argc, char ** argv)
 	return 0;
 }
 
+/* ipmi_raw_help() - print 'raw' help text
+ *
+ * returns void
+ */
+void
+ipmi_raw_help()
+{
+	lprintf(LOG_NOTICE, "RAW Commands:  raw <netfn> <cmd> [data]");
+	print_valstr(ipmi_netfn_vals, "Network Function Codes", LOG_NOTICE);
+	lprintf(LOG_NOTICE, "(can also use raw hex values)");
+} /* ipmi_raw_help() */
+
 int
 ipmi_raw_main(struct ipmi_intf * intf, int argc, char ** argv)
 {
 	struct ipmi_rs * rsp;
 	struct ipmi_rq req;
 	uint8_t netfn, cmd, lun;
+	uint16_t netfn_tmp = 0;
 	int i;
 	uint8_t data[256];
 
-	if (argc < 2 || strncmp(argv[0], "help", 4) == 0) {
-		lprintf(LOG_NOTICE, "RAW Commands:  raw <netfn> <cmd> [data]");
-		print_valstr(ipmi_netfn_vals, "Network Function Codes", LOG_NOTICE);
-		lprintf(LOG_NOTICE, "(can also use raw hex values)");
-		return -1;
+	if (argc == 1 && strncmp(argv[0], "help", 4) == 0) {
+		ipmi_raw_help();
+		return 0;
+	}
+	else if (argc < 2) {
+		lprintf(LOG_ERR, "Not enough parameters given.");
+		ipmi_raw_help();
+		return (-1);
 	}
 	else if (argc > sizeof(data))
 	{
@@ -300,12 +341,20 @@ ipmi_raw_main(struct ipmi_intf * intf, int argc, char ** argv)
 	ipmi_intf_session_set_retry(intf, 1);
 
 	lun = intf->target_lun;
-	netfn = str2val(argv[0], ipmi_netfn_vals);
-	if (netfn == 0xff) {
-		netfn = (uint8_t)strtol(argv[0], NULL, 0);
+	netfn_tmp = str2val(argv[0], ipmi_netfn_vals);
+	if (netfn_tmp == 0xff) {
+		if (is_valid_param(argv[0], &netfn, "netfn") != 0)
+			return (-1);
+	} else {
+		if (netfn_tmp >= UINT8_MAX) {
+			lprintf(LOG_ERR, "Given netfn \"%s\" is out of range.", argv[0]);
+			return (-1);
+		}
+		netfn = netfn_tmp;
 	}
 
-	cmd = (uint8_t)strtol(argv[1], NULL, 0);
+	if (is_valid_param(argv[1], &cmd, "command") != 0)
+		return (-1);
 
 	memset(data, 0, sizeof(data));
 	memset(&req, 0, sizeof(req));
@@ -315,7 +364,11 @@ ipmi_raw_main(struct ipmi_intf * intf, int argc, char ** argv)
 	req.msg.data = data;
 
 	for (i=2; i<argc; i++) {
-		uint8_t val = (uint8_t)strtol(argv[i], NULL, 0);
+		uint8_t val = 0;
+
+		if (is_valid_param(argv[i], &val, "data") != 0)
+			return (-1);
+
 		req.msg.data[i-2] = val;
 		req.msg.data_len++;
 	}
@@ -355,3 +408,25 @@ ipmi_raw_main(struct ipmi_intf * intf, int argc, char ** argv)
 
 	return 0;
 }
+
+/* is_valid_param - 
+ *
+ * @input_param: string to convert from
+ * @uchr_ptr: pointer where to store converted value
+ * @label: string used in error message
+ *
+ * returns   0  if parameter is valid
+ * returns (-1) if parameter is invalid/on error
+ */
+int
+is_valid_param(const char *input_param, uint8_t *uchr_ptr, const char *label) {
+	if (input_param == NULL || label == NULL) {
+		lprintf(LOG_ERROR, "ERROR: NULL pointer passed.");
+		return (-1);
+	}
+	if (str2uchar(input_param, uchr_ptr) == 0)
+		return 0;
+
+	lprintf(LOG_ERR, "Given %s \"%s\" is invalid.", label, input_param);
+	return (-1);
+}
