@@ -31,11 +31,11 @@
  */
 
 #include <string.h>
-#include <stdlib.h> /* for strtoul, to parse hexadecimal values  */
 #include <math.h>
 #define __USE_XOPEN /* glibc2 needs this for strptime */
 #include <time.h>
 #include <ctype.h>
+#include <errno.h>
 
 #include <ipmitool/helper.h>
 #include <ipmitool/log.h>
@@ -155,10 +155,12 @@ int ipmi_sel_oem_init(const char * filename)
 				for (k=3; k<17; k++) {
 					if (sel_oem_msg[j].value[SEL_BYTE(k)] == -3) {
 						free(sel_oem_msg[j].string[SEL_BYTE(k)]);
+						sel_oem_msg[j].string[SEL_BYTE(k)] = NULL;
 					}
 				}
 			}
-			free (sel_oem_msg);
+			free(sel_oem_msg);
+			sel_oem_msg = NULL;
 			return -1;
 		}
 
@@ -307,14 +309,14 @@ ipmi_get_oem(struct ipmi_intf * intf)
 		return IPMI_OEM_UNKNOWN;
 	}
 	if (rsp->ccode > 0) {
-		lprintf(LOG_ERR, "Get Device ID command failed: %s",
-			val2str(rsp->ccode, completion_code_vals));
+		lprintf(LOG_ERR, "Get Device ID command failed: %#x %s",
+			rsp->ccode, val2str(rsp->ccode, completion_code_vals));
 		return IPMI_OEM_UNKNOWN;
 	}
 
 	devid = (struct ipm_devid_rsp *) rsp->data;
 
-	lprintf(LOG_DEBUG,"Iana: 0x%u",
+	lprintf(LOG_DEBUG,"Iana: %u",
            IPM_DEV_MANUFACTURER_ID(devid->manufacturer_id));
 
 	return  IPM_DEV_MANUFACTURER_ID(devid->manufacturer_id);
@@ -327,6 +329,7 @@ ipmi_sel_add_entry(struct ipmi_intf * intf, struct sel_event_record * rec)
 	struct ipmi_rs * rsp;
 	struct ipmi_rq req;
 
+	memset(&req, 0, sizeof(req));
 	req.msg.netfn = IPMI_NETFN_STORAGE;
 	req.msg.cmd = IPMI_CMD_ADD_SEL_ENTRY;
 	req.msg.data = (unsigned char *)rec;
@@ -396,7 +399,9 @@ ipmi_sel_add_entries_fromfile(struct ipmi_intf * intf, const char * filename)
 			if (i == 7)
 				break;
 			j = i++;
-			rqdata[j] = (uint8_t)strtol(tok, NULL, 0);
+			if (str2uchar(tok, &rqdata[j]) != 0) {
+				break;
+			}
 			tok = strtok(NULL, " ");
 		}
 		if (i < 7) {
@@ -609,12 +614,20 @@ char * get_dell_evt_desc(struct ipmi_intf * intf, struct sel_event_record * rec)
 				if (NULL == rsp) 
 				{
 					lprintf(LOG_ERR, " Error getting system info");
+					if (desc != NULL) {
+						free(desc);
+						desc = NULL;
+					}
 					return NULL;
 				} 
 				else if (rsp->ccode > 0)
 				{
 					lprintf(LOG_ERR, " Error getting system info: %s",
 						val2str(rsp->ccode, completion_code_vals));
+					if (desc != NULL) {
+						free(desc);
+						desc = NULL;
+					}
 					return NULL;
 				}
 				version = rsp->data[4];
@@ -1166,6 +1179,7 @@ ipmi_get_event_desc(struct ipmi_intf * intf, struct sel_event_record * rec, char
 			if (sfx) {
 				sprintf(*desc, "%s (%s)", evt->desc, sfx);
 				free(sfx);
+				sfx = NULL;
 			} else {
 				sprintf(*desc, "%s", evt->desc);
 			}
@@ -1211,6 +1225,7 @@ ipmi_get_event_desc(struct ipmi_intf * intf, struct sel_event_record * rec, char
 		sprintf(*desc, "(%s)",sfx);		
      	}
 		free(sfx);
+		sfx = NULL;
 	}
 }
 
@@ -1396,7 +1411,7 @@ ipmi_sel_get_info(struct ipmi_intf * intf)
 		printf("Alloc Unit Size  : %d\n", buf2short(rsp->data + 2));
 		printf("# Free Units     : %d\n", buf2short(rsp->data + 4));
 		printf("Largest Free Blk : %d\n", buf2short(rsp->data + 6));
-		printf("Max Record Size  : %d\n", rsp->data[7]);
+		printf("Max Record Size  : %d\n", rsp->data[8]);
 	}
 	return 0;
 }
@@ -1536,8 +1551,10 @@ ipmi_sel_print_event_file(struct ipmi_intf * intf, struct sel_event_record * evt
 		evt->sel_type.standard_type.sensor_num,
 		(description != NULL) ? description : "Unknown");
 
-	if (description != NULL)
+	if (description != NULL) {
 		free(description);
+		description = NULL;
+	}
 }
 
 void
@@ -1703,18 +1720,19 @@ ipmi_sel_print_std_entry(struct ipmi_intf * intf, struct sel_event_record * evt)
 	if (description) {
 		printf("%s", description);
 		free(description);
+		description = NULL;
 	}
 
-	if (evt->sel_type.standard_type.event_type == 0x6f) {
-		if (csv_output)
-			printf(",");
-		else
-			printf(" | ");
+	if (csv_output) {
+		printf(",");
+	} else {
+		printf(" | ");
+	}
 
-		if (evt->sel_type.standard_type.event_dir)
-			printf("Deasserted");
-		else
-			printf("Asserted");
+	if (evt->sel_type.standard_type.event_dir) {
+		printf("Deasserted");
+	} else {
+		printf("Asserted");
 	}
 
 	if (sdr != NULL && evt->sel_type.standard_type.event_type == 1) {
@@ -1723,6 +1741,7 @@ ipmi_sel_print_std_entry(struct ipmi_intf * intf, struct sel_event_record * evt)
 		 */
 		float trigger_reading = 0.0;
 		float threshold_reading = 0.0;
+		uint8_t threshold_reading_provided = 0;
 
 		/* trigger reading in event data byte 2 */
 		if (((evt->sel_type.standard_type.event_data[0] >> 6) & 3) == 1) {
@@ -1734,6 +1753,7 @@ ipmi_sel_print_std_entry(struct ipmi_intf * intf, struct sel_event_record * evt)
 		if (((evt->sel_type.standard_type.event_data[0] >> 4) & 3) == 1) {
 			threshold_reading = sdr_convert_sensor_reading(
 				sdr->record.full, evt->sel_type.standard_type.event_data[2]);
+			threshold_reading_provided = 1;
 		}
 
 		if (csv_output)
@@ -1741,16 +1761,19 @@ ipmi_sel_print_std_entry(struct ipmi_intf * intf, struct sel_event_record * evt)
 		else
 			printf(" | ");
 		
-		printf("Reading %.*f %s Threshold %.*f %s",
-		       (trigger_reading==(int)trigger_reading) ? 0 : 2,
-		       trigger_reading,
-		       ((evt->sel_type.standard_type.event_data[0] & 0xf) % 2) ? ">" : "<",
-		       (threshold_reading==(int)threshold_reading) ? 0 : 2,
-		       threshold_reading,
-		       ipmi_sdr_get_unit_string(sdr->record.common->unit.pct,
+		printf("Reading %.*f",
+				(trigger_reading==(int)trigger_reading) ? 0 : 2,
+				trigger_reading);
+		if (threshold_reading_provided) {
+			printf(" %s Threshold %.*f %s",
+					((evt->sel_type.standard_type.event_data[0] & 0xf) % 2) ? ">" : "<",
+					(threshold_reading==(int)threshold_reading) ? 0 : 2,
+					threshold_reading,
+					ipmi_sdr_get_unit_string(sdr->record.common->unit.pct,
 						sdr->record.common->unit.modifier,
 						sdr->record.common->unit.type.base,
 						sdr->record.common->unit.type.modifier));
+		}
 	}
 	else if (evt->sel_type.standard_type.event_type == 0x6f) {
 		/*
@@ -1861,6 +1884,7 @@ ipmi_sel_print_std_entry_verbose(struct ipmi_intf * intf, struct sel_event_recor
 	printf(" Description           : %s\n",
                description ? description : "");
         free(description);
+				description = NULL;
 
 	printf("\n");
 }
@@ -2015,6 +2039,7 @@ ipmi_sel_print_extended_entry_verbose(struct ipmi_intf * intf, struct sel_event_
 	printf(" Description           : %s\n",
                description ? description : "");
         free(description);
+				description = NULL;
 
 	printf("\n");
 }
@@ -2160,165 +2185,230 @@ ipmi_sel_save_entries(struct ipmi_intf * intf, int count, const char * savefile)
  *        -1 on error
  */
 static int
-ipmi_sel_interpret(struct ipmi_intf * intf, unsigned long iana, const char * readfile, const char *format)
+ipmi_sel_interpret(struct ipmi_intf *intf, unsigned long iana,
+		const char *readfile, const char *format)
 {
-	FILE* fp = 0;
-	int status = 0;
-
+	FILE *fp = 0;
 	struct sel_event_record evt;
-	char *buffer;
-
-
-	/* since the interface is not used, iana is taken from the command line */
+	char *buffer = NULL;
+	char *cursor = NULL;
+	int status = 0;
+	/* since the interface is not used, iana is taken from
+	 * the command line
+	 */
 	sel_iana = iana;
-
-
-	if( strncmp("pps",format,3) == 0 ) {
-			/* Parser for the following format */
-			/* 
-             0x001F: Event: at Mar 27 06:41:10 2007;from:(0x9a,0,7);				\ 
-                    sensor:(0xc3,119); event:0x6f(asserted): 0xA3 0x00 0x88   
-            commonly found in PPS shelf managers 
-            Supports a tweak for hotswap events that are already interpreted.
-         */
+	if (strncmp("pps", format, 3) == 0) {
+		/* Parser for the following format */
+		/* 0x001F: Event: at Mar 27 06:41:10 2007;from:(0x9a,0,7);
+		 * sensor:(0xc3,119); event:0x6f(asserted): 0xA3 0x00 0x88
+		 * commonly found in PPS shelf managers
+		 * Supports a tweak for hotswap events that are already interpreted.
+		 */
 		fp = ipmi_open_file(readfile, 0);
-		if (fp){
-			buffer = (char*)malloc((size_t)256);
-			if( buffer != NULL ) {		
-				do {
-					/* Only allow complete lines to be parsed,hardcoded maximum
-                  line length */
-
-					if( fgets(buffer, 256, fp) != NULL ){
-						if( strlen(buffer) < 256 )	{
-							char *cursor = buffer;
-
-							evt.record_type= 2; /* assume normal "System" event */
-							evt.record_id=
-								strtol((const char*)cursor, (char **)NULL,16);
-							
-							evt.sel_type.standard_type.evm_rev = 4;
-
-							/* FIXME: convert*/ evt.sel_type.standard_type.timestamp;
-
-							/* skip timestamp */
-							cursor = index((const char*)cursor,';');
-							cursor++;
-
-							/* FIXME: parse originator */
-							evt.sel_type.standard_type.gen_id = 0x0020;
-
-							/* skip  originator info */
-							cursor = index((const char*)cursor,';');
-							cursor++;
-
-							/* Get sensor type */
-							cursor = index((const char*)cursor,'(');
-							cursor++;
-
-							evt.sel_type.standard_type.sensor_type=
-								strtol((const char*)cursor, (char **)NULL,16);
-
-							cursor = index((const char*)cursor,',');
-							cursor++;
-
-							evt.sel_type.standard_type.sensor_num=
-								strtol((const char*)cursor, (char **)NULL,10);
-
-							/* skip  to event type  info */
-							cursor = index((const char*)cursor,':');
-							cursor++;
-
-							evt.sel_type.standard_type.event_type=
-								strtol((const char*)cursor, (char **)NULL,16);
-
-							/* skip  to event dir  info */
-							cursor = index((const char*)cursor,'(');
-							cursor++;
-							if( *cursor == 'a' ) {
-								evt.sel_type.standard_type.event_dir = 0;
-							}
-							else {
-								evt.sel_type.standard_type.event_dir = 1;
-							}
-							/* skip  to data info */
-							cursor = index((const char*)cursor,' ');
-							cursor++;
-
-							if( evt.sel_type.standard_type.sensor_type == 0xF0 ) {		
-								
-								/* got to FRU id */
-								while( !isdigit(*cursor) ){
-									cursor++;
-								}
-                        /* store FRUid */
-								evt.sel_type.standard_type.event_data[2] =
-									strtol(cursor, (char **)NULL,10);
-
-								/* Get to previous state */
-								cursor = index((const char*)cursor,'M');
-								cursor++;
-		
-
-								/* Set previous state */
-								evt.sel_type.standard_type.event_data[1] =
-									strtol(cursor, (char **)NULL,10);
-
-								/* Get to current state */
-								cursor = index((const char*)cursor,'M');
-								cursor++;
-
-
-								/* Set current state */
-								evt.sel_type.standard_type.event_data[0] =
-									0xA0 | strtol(cursor, (char **)NULL,10) ;
-
-								/* skip  to cause */
-								cursor = index((const char*)cursor,'=');
-								cursor++;							
-								evt.sel_type.standard_type.event_data[1] |=
-									(strtol(cursor, (char **)NULL,16))<<4 ; 
-							}else if( *cursor == '0' ) { 
-
-								evt.sel_type.standard_type.event_data[0]=
-									strtol((const char*)cursor, (char **)NULL,16);
-
-								cursor = index((const char*)cursor,' ');
-								cursor++;
-
-								evt.sel_type.standard_type.event_data[1]=
-									strtol((const char*)cursor, (char **)NULL,16);
-
-								cursor = index((const char*)cursor,' ');
-								cursor++;
-							
-								evt.sel_type.standard_type.event_data[2]=
-									strtol((const char*)cursor, (char **)NULL,16);
-							}else {
-								lprintf(LOG_ERR, "ipmitool: can't guess format.");
-							}
-							
-							/* parse the PPS line into a sel_event_record */
-							if (verbose) {
-								ipmi_sel_print_std_entry_verbose(intf, &evt);
-							}
-							else {
-								ipmi_sel_print_std_entry(intf, &evt);
-							}
-						}else{   /* length didn't fit */
-							lprintf(LOG_ERR, "ipmitool: invalid entry found in file.");
-						}	
-					}else{ /* fgets failed (or reached end of file) */
-						status = -1;
-					}
-				}while (status == 0); /* until file is completely read */
-				free(buffer);
-			}	/* if memory allocation succeeded */
+		if (fp == NULL) {
+			lprintf(LOG_ERR, "Failed to open file '%s' for reading.",
+					readfile);
+			return (-1);
+		}
+		buffer = (char *)malloc((size_t)256);
+		if (buffer == NULL) {
+			lprintf(LOG_ERR, "ipmitool: malloc failure");
 			fclose(fp);
-		} 	/* if file open succeeded */
-	}	/* if format is known */
+			return (-1);
+		}
+		do {
+			/* Only allow complete lines to be parsed,
+			 * hardcoded maximum line length
+			 */
+			if (fgets(buffer, 256, fp) == NULL) {
+				status = (-1);
+				break;
+			}
+			if (strlen(buffer) > 255) {
+				lprintf(LOG_ERR, "ipmitool: invalid entry found in file.");
+				continue;
+			}
+			cursor = buffer;
+			/* assume normal "System" event */
+			evt.record_type = 2;
+			errno = 0;
+			evt.record_id = strtol((const char *)cursor, (char **)NULL, 16);
+			if (errno != 0) {
+				lprintf(LOG_ERR, "Invalid record ID.");
+				status = (-1);
+				break;
+			}	
+			evt.sel_type.standard_type.evm_rev = 4;
 
-	return status;	
+			/* FIXME: convert*/
+			evt.sel_type.standard_type.timestamp;
+
+			/* skip timestamp */
+			cursor = index((const char *)cursor, ';');
+			cursor++;
+
+			/* FIXME: parse originator */
+			evt.sel_type.standard_type.gen_id = 0x0020;
+
+			/* skip  originator info */
+			cursor = index((const char *)cursor, ';');
+			cursor++;
+
+			/* Get sensor type */
+			cursor = index((const char *)cursor, '(');
+			cursor++;
+
+			errno = 0;
+			evt.sel_type.standard_type.sensor_type =
+				strtol((const char *)cursor, (char **)NULL, 16);
+			if (errno != 0) {
+				lprintf(LOG_ERR, "Invalid Sensor Type.");
+				status = (-1);
+				break;
+			}	
+			cursor = index((const char *)cursor, ',');
+			cursor++;
+
+			errno = 0;
+			evt.sel_type.standard_type.sensor_num =
+				strtol((const char *)cursor, (char **)NULL, 10);
+			if (errno != 0) {
+				lprintf(LOG_ERR, "Invalid Sensor Number.");
+				status = (-1);
+				break;
+			}	
+
+			/* skip  to event type  info */
+			cursor = index((const char *)cursor, ':');
+			cursor++;
+
+			errno = 0;
+			evt.sel_type.standard_type.event_type=
+				strtol((const char *)cursor, (char **)NULL, 16);
+			if (errno != 0) {
+				lprintf(LOG_ERR, "Invalid Event Type.");
+				status = (-1);
+				break;
+			}	
+
+			/* skip  to event dir  info */
+			cursor = index((const char *)cursor, '(');
+			cursor++;
+			if (*cursor == 'a') {
+				evt.sel_type.standard_type.event_dir = 0;
+			} else {
+				evt.sel_type.standard_type.event_dir = 1;
+			}
+			/* skip  to data info */
+			cursor = index((const char *)cursor, ' ');
+			cursor++;
+
+			if (evt.sel_type.standard_type.sensor_type == 0xF0) {
+				/* got to FRU id */
+				while (!isdigit(*cursor)) {
+					cursor++;
+				}
+				/* store FRUid */
+				errno = 0;
+				evt.sel_type.standard_type.event_data[2] =
+					strtol(cursor, (char **)NULL, 10);
+				if (errno != 0) {
+					lprintf(LOG_ERR, "Invalid Event Data#2.");
+					status = (-1);
+					break;
+				}	
+
+				/* Get to previous state */
+				cursor = index((const char *)cursor, 'M');
+				cursor++;
+
+				/* Set previous state */
+				errno = 0;
+				evt.sel_type.standard_type.event_data[1] =
+					strtol(cursor, (char **)NULL, 10);
+				if (errno != 0) {
+					lprintf(LOG_ERR, "Invalid Event Data#1.");
+					status = (-1);
+					break;
+				}	
+
+				/* Get to current state */
+				cursor = index((const char *)cursor, 'M');
+				cursor++;
+
+				/* Set current state */
+				errno = 0;
+				evt.sel_type.standard_type.event_data[0] =
+					0xA0 | strtol(cursor, (char **)NULL, 10);
+				if (errno != 0) {
+					lprintf(LOG_ERR, "Invalid Event Data#0.");
+					status = (-1);
+					break;
+				}	
+
+				/* skip  to cause */
+				cursor = index((const char *)cursor, '=');
+				cursor++;
+				errno = 0;
+				evt.sel_type.standard_type.event_data[1] |=
+					(strtol(cursor, (char **)NULL, 16)) << 4;
+				if (errno != 0) {
+					lprintf(LOG_ERR, "Invalid Event Data#1.");
+					status = (-1);
+					break;
+				}	
+			} else if (*cursor == '0') {
+				errno = 0;
+				evt.sel_type.standard_type.event_data[0] =
+					strtol((const char *)cursor, (char **)NULL, 16);
+				if (errno != 0) {
+					lprintf(LOG_ERR, "Invalid Event Data#0.");
+					status = (-1);
+					break;
+				}	
+				cursor = index((const char *)cursor, ' ');
+				cursor++;
+
+				errno = 0;
+				evt.sel_type.standard_type.event_data[1] =
+					strtol((const char *)cursor, (char **)NULL, 16);
+				if (errno != 0) {
+					lprintf(LOG_ERR, "Invalid Event Data#1.");
+					status = (-1);
+					break;
+				}	
+
+				cursor = index((const char *)cursor, ' ');
+				cursor++;
+
+				errno = 0;
+				evt.sel_type.standard_type.event_data[2] =
+					strtol((const char *)cursor, (char **)NULL, 16);
+				if (errno != 0) {
+					lprintf(LOG_ERR, "Invalid Event Data#2.");
+					status = (-1);
+					break;
+				}	
+			} else {
+				lprintf(LOG_ERR, "ipmitool: can't guess format.");
+			}
+			/* parse the PPS line into a sel_event_record */
+			if (verbose) {
+				ipmi_sel_print_std_entry_verbose(intf, &evt);
+			} else {
+				ipmi_sel_print_std_entry(intf, &evt);
+			}
+			cursor = NULL;
+		} while (status == 0); /* until file is completely read */
+		cursor = NULL;
+		free(buffer);
+		buffer = NULL;
+		fclose(fp);
+	} else {
+		lprintf(LOG_ERR, "Given format '%s' is unknown.", format);
+		status = (-1);
+	}
+	return status;
 }
 
 
@@ -2461,7 +2551,7 @@ ipmi_sel_set_time(struct ipmi_intf * intf, const char * time_string)
 {
 	struct ipmi_rs     * rsp;
 	struct ipmi_rq       req;
-	struct tm            tm;
+	struct tm            tm = {0};
 	time_t               t;
 	uint32_t	     timei;
 	const char *         time_format = "%m/%d/%Y %H:%M:%S";
@@ -2480,6 +2570,7 @@ ipmi_sel_set_time(struct ipmi_intf * intf, const char * time_string)
 			lprintf(LOG_ERR, "Specified time could not be parsed");
 			return -1;
 		}
+		tm.tm_isdst = (-1); /* look up DST information */
 		t = mktime(&tm);
 		if (t < 0) {
 			lprintf(LOG_ERR, "Specified time could not be parsed");
@@ -2489,13 +2580,14 @@ ipmi_sel_set_time(struct ipmi_intf * intf, const char * time_string)
 
 	{
 		//modify UTC time to local time expressed in number of seconds from 1/1/70 0:0:0 1970 GMT
-		struct tm * tm_tmp;
+		struct tm * tm_tmp = {0};
 		int gt_year,gt_yday,gt_hour,lt_year,lt_yday,lt_hour;
 		int delta_hour;
 		tm_tmp=gmtime(&t);
 		gt_year=tm_tmp->tm_year;
 		gt_yday=tm_tmp->tm_yday;
 		gt_hour=tm_tmp->tm_hour;
+		memset(&*tm_tmp, 0, sizeof(struct tm));
 		tm_tmp=localtime(&t);
 		lt_year=tm_tmp->tm_year;
 		lt_yday=tm_tmp->tm_yday;
@@ -2601,6 +2693,12 @@ ipmi_sel_delete(struct ipmi_intf * intf, int argc, char ** argv)
 	for (; argc != 0; argc--)
 	{
 		id = (uint16_t) strtoul(argv[argc-1], NULL, 0);
+		if (str2ushort(argv[argc-1], &id) != 0) {
+			lprintf(LOG_ERR, "Given SEL ID '%s' is invalid.",
+					argv[argc-1]);
+			rc = (-1);
+			continue;
+		}
 		msg_data[2] = id & 0xff;
 		msg_data[3] = id >> 8;
 
@@ -2650,7 +2748,12 @@ ipmi_sel_show_entry(struct ipmi_intf * intf, int argc, char ** argv)
 	}
 
 	for (i=0; i<argc; i++) {
-		id = (uint16_t)strtol(argv[i], NULL, 0);
+		if (str2ushort(argv[i], &id) != 0) {
+			lprintf(LOG_ERR, "Given SEL ID '%s' is invalid.",
+					argv[i]);
+			rc = (-1);
+			continue;
+		}
 
 		lprintf(LOG_DEBUG, "Looking up SEL entry 0x%x", id);
 
@@ -2707,17 +2810,6 @@ ipmi_sel_show_entry(struct ipmi_intf * intf, int argc, char ** argv)
 	return rc;
 }
 
-static int make_int(const char *str, int *value)
-{
-	char *tmp=NULL;
-	*value = strtol(str,&tmp,0);
-	if ( tmp-str != strlen(str) )
-	{
-		return -1;
-	}
-	return 0;
-}
-
 int ipmi_sel_main(struct ipmi_intf * intf, int argc, char ** argv)
 {
 	int rc = 0;
@@ -2728,11 +2820,17 @@ int ipmi_sel_main(struct ipmi_intf * intf, int argc, char ** argv)
 		lprintf(LOG_ERR, "SEL Commands:  "
 				"info clear delete list elist get add time save readraw writeraw interpret");
 	else if (strncmp(argv[0], "interpret", 9) == 0) {
+		uint32_t iana = 0;
 		if (argc < 4) {
 			lprintf(LOG_NOTICE, "usage: sel interpret iana filename format(pps)");
 			return 0;
 		}
-		rc = ipmi_sel_interpret(intf, atol(argv[1]),argv[2],argv[3]);
+		if (str2uint(argv[1], &iana) != 0) {
+			lprintf(LOG_ERR, "Given IANA '%s' is invalid.",
+					argv[1]);
+			return (-1);
+		}
+		rc = ipmi_sel_interpret(intf, iana, argv[2], argv[3]);
 	}
 	else if (strncmp(argv[0], "info", 4) == 0)
 		rc = ipmi_sel_get_info(intf);
@@ -2805,7 +2903,7 @@ int ipmi_sel_main(struct ipmi_intf * intf, int argc, char ** argv)
 		}
 
 		if (countstr) {
-			if (make_int(countstr,&count) < 0) {
+			if (str2int(countstr, &count) != 0) {
 				lprintf(LOG_ERR, "Numeric argument required; got '%s'",
 					countstr);
 				return -1;
