@@ -42,6 +42,8 @@
 #include <ipmitool/ipmi_sensor.h>
 
 extern int verbose;
+void printf_sensor_get_usage();
+
 // Macro's for Reading the current sensor Data.
 #define SCANNING_DISABLED	0x40
 #define READING_UNAVAILABLE	0x20
@@ -70,6 +72,7 @@ ipmi_sensor_get_sensor_reading_factors(
 
 	memset(&req, 0, sizeof(req));
 	req.msg.netfn = IPMI_NETFN_SE;
+	req.msg.lun = sensor->cmn.keys.lun;
 	req.msg.cmd   = GET_SENSOR_FACTORS;
 	req.msg.data  = req_data;
 	req.msg.data_len = sizeof(req_data);
@@ -102,12 +105,13 @@ struct ipmi_rs *
 ipmi_sensor_set_sensor_thresholds(struct ipmi_intf *intf,
 				  uint8_t sensor,
 				  uint8_t threshold, uint8_t setting,
-				  uint8_t target, uint8_t lun)
+				  uint8_t target, uint8_t lun, uint8_t channel)
 {
 	struct ipmi_rq req;
 	static struct sensor_set_thresh_rq set_thresh_rq;
 	struct ipmi_rs *rsp;
-	uint8_t save_addr;
+	uint32_t save_addr = 0;
+	uint32_t save_channel;
 
 	memset(&set_thresh_rq, 0, sizeof (set_thresh_rq));
 	set_thresh_rq.sensor_num = sensor;
@@ -127,17 +131,24 @@ ipmi_sensor_set_sensor_thresholds(struct ipmi_intf *intf,
 	else
 		return NULL;
 
-	save_addr = intf->target_addr;
-	intf->target_addr = target;
-
+	if (BRIDGE_TO_SENSOR(intf, target, channel)) {
+		save_addr = intf->target_addr;
+		intf->target_addr = target;
+		save_channel = intf->target_channel;
+		intf->target_channel = channel;
+	}
 	memset(&req, 0, sizeof (req));
 	req.msg.netfn = IPMI_NETFN_SE;
+	req.msg.lun = lun;
 	req.msg.cmd = SET_SENSOR_THRESHOLDS;
 	req.msg.data = (uint8_t *) & set_thresh_rq;
 	req.msg.data_len = sizeof (set_thresh_rq);
 
 	rsp = intf->sendrecv(intf, &req);
-	intf->target_addr = save_addr;
+	if (save_addr) {
+		intf->target_addr = save_addr;
+		intf->target_channel = save_channel;
+	}
 	return rsp;
 }
 
@@ -436,6 +447,7 @@ ipmi_sensor_list(struct ipmi_intf *intf)
 			break;
 		}
 		free(rec);
+		rec = NULL;
 
 		/* fix for CR6604909: */
 		/* mask failure of individual reads in sensor list command */
@@ -460,12 +472,12 @@ static const struct valstr threshold_vals[] = {
 static int
 __ipmi_sensor_set_threshold(struct ipmi_intf *intf,
 			    uint8_t num, uint8_t mask, uint8_t setting,
-			    uint8_t target, uint8_t lun)
+			    uint8_t target, uint8_t lun, uint8_t channel)
 {
 	struct ipmi_rs *rsp;
 
 	rsp = ipmi_sensor_set_sensor_thresholds(intf, num, mask, setting,
-				  target, lun);
+				  target, lun, channel);
 
 	if (rsp == NULL) {
 		lprintf(LOG_ERR, "Error setting threshold");
@@ -557,9 +569,21 @@ ipmi_sensor_set_threshold(struct ipmi_intf *intf, int argc, char **argv)
 			return -1;
 		}
 		allUpper = 1;
-		setting1 = (double) strtod(argv[2], NULL);
-		setting2 = (double) strtod(argv[3], NULL);
-		setting3 = (double) strtod(argv[4], NULL);
+		if (str2double(argv[2], &setting1) != 0) {
+			lprintf(LOG_ERR, "Given unc '%s' is invalid.",
+					argv[2]);
+			return (-1);
+		}
+		if (str2double(argv[3], &setting2) != 0) {
+			lprintf(LOG_ERR, "Given ucr '%s' is invalid.",
+					argv[3]);
+			return (-1);
+		}
+		if (str2double(argv[4], &setting3) != 0) {
+			lprintf(LOG_ERR, "Given unr '%s' is invalid.",
+					argv[4]);
+			return (-1);
+		}
 	} else if (strncmp(thresh, "lower", 5) == 0) {
 		if (argc < 5) {
 			lprintf(LOG_ERR,
@@ -567,11 +591,22 @@ ipmi_sensor_set_threshold(struct ipmi_intf *intf, int argc, char **argv)
 			return -1;
 		}
 		allLower = 1;
-		setting1 = (double) strtod(argv[2], NULL);
-		setting2 = (double) strtod(argv[3], NULL);
-		setting3 = (double) strtod(argv[4], NULL);
+		if (str2double(argv[2], &setting1) != 0) {
+			lprintf(LOG_ERR, "Given lnc '%s' is invalid.",
+					argv[2]);
+			return (-1);
+		}
+		if (str2double(argv[3], &setting2) != 0) {
+			lprintf(LOG_ERR, "Given lcr '%s' is invalid.",
+					argv[3]);
+			return (-1);
+		}
+		if (str2double(argv[4], &setting3) != 0) {
+			lprintf(LOG_ERR, "Given lnr '%s' is invalid.",
+					argv[4]);
+			return (-1);
+		}
 	} else {
-		setting1 = (double) atof(argv[2]);
 		if (strncmp(thresh, "unr", 3) == 0)
 			settingMask = UPPER_NON_RECOV_SPECIFIED;
 		else if (strncmp(thresh, "ucr", 3) == 0)
@@ -590,6 +625,12 @@ ipmi_sensor_set_threshold(struct ipmi_intf *intf, int argc, char **argv)
 				thresh, id);
 			return -1;
 		}
+		if (str2double(argv[2], &setting1) != 0) {
+			lprintf(LOG_ERR,
+					"Given %s threshold value '%s' is invalid.",
+					thresh, argv[2]);
+			return (-1);
+		}
 	}
 
 	printf("Locating sensor record '%s'...\n", id);
@@ -622,7 +663,8 @@ ipmi_sensor_set_threshold(struct ipmi_intf *intf, int argc, char **argv)
 						  sensor_num, settingMask,
 						  __ipmi_sensor_threshold_value_to_raw(sdr->record.full, setting1),
 						  sdr->record.common->keys.owner_id,
-						  sdr->record.common->keys.lun);
+						  sdr->record.common->keys.lun,
+						  sdr->record.common->keys.channel);
 
 		settingMask = UPPER_CRIT_SPECIFIED;
 		printf("Setting sensor \"%s\" %s threshold to %.3f\n",
@@ -633,7 +675,8 @@ ipmi_sensor_set_threshold(struct ipmi_intf *intf, int argc, char **argv)
 						  sensor_num, settingMask,
 						  __ipmi_sensor_threshold_value_to_raw(sdr->record.full, setting2),
 						  sdr->record.common->keys.owner_id,
-						  sdr->record.common->keys.lun);
+						  sdr->record.common->keys.lun,
+						  sdr->record.common->keys.channel);
 
 		settingMask = UPPER_NON_RECOV_SPECIFIED;
 		printf("Setting sensor \"%s\" %s threshold to %.3f\n",
@@ -644,7 +687,8 @@ ipmi_sensor_set_threshold(struct ipmi_intf *intf, int argc, char **argv)
 						  sensor_num, settingMask,
 						  __ipmi_sensor_threshold_value_to_raw(sdr->record.full, setting3),
 						  sdr->record.common->keys.owner_id,
-						  sdr->record.common->keys.lun);
+						  sdr->record.common->keys.lun,
+						  sdr->record.common->keys.channel);
 	} else if (allLower) {
 		settingMask = LOWER_NON_RECOV_SPECIFIED;
 		printf("Setting sensor \"%s\" %s threshold to %.3f\n",
@@ -655,7 +699,8 @@ ipmi_sensor_set_threshold(struct ipmi_intf *intf, int argc, char **argv)
 						  sensor_num, settingMask,
 						  __ipmi_sensor_threshold_value_to_raw(sdr->record.full, setting1),
 						  sdr->record.common->keys.owner_id,
-						  sdr->record.common->keys.lun);
+						  sdr->record.common->keys.lun,
+						  sdr->record.common->keys.channel);
 
 		settingMask = LOWER_CRIT_SPECIFIED;
 		printf("Setting sensor \"%s\" %s threshold to %.3f\n",
@@ -666,7 +711,8 @@ ipmi_sensor_set_threshold(struct ipmi_intf *intf, int argc, char **argv)
 						  sensor_num, settingMask,
 						  __ipmi_sensor_threshold_value_to_raw(sdr->record.full, setting2),
 						  sdr->record.common->keys.owner_id,
-						  sdr->record.common->keys.lun);
+						  sdr->record.common->keys.lun,
+						  sdr->record.common->keys.channel);
 
 		settingMask = LOWER_NON_CRIT_SPECIFIED;
 		printf("Setting sensor \"%s\" %s threshold to %.3f\n",
@@ -677,7 +723,8 @@ ipmi_sensor_set_threshold(struct ipmi_intf *intf, int argc, char **argv)
 						  sensor_num, settingMask,
 						  __ipmi_sensor_threshold_value_to_raw(sdr->record.full, setting3),
 						  sdr->record.common->keys.owner_id,
-						  sdr->record.common->keys.lun);
+						  sdr->record.common->keys.lun,
+						  sdr->record.common->keys.channel);
 	} else {
 
 	/*
@@ -775,7 +822,8 @@ ipmi_sensor_set_threshold(struct ipmi_intf *intf, int argc, char **argv)
 						  sensor_num, settingMask,
 						  __ipmi_sensor_threshold_value_to_raw(sdr->record.full, setting1),
 						  sdr->record.common->keys.owner_id,
-						  sdr->record.common->keys.lun);
+						  sdr->record.common->keys.lun,
+						  sdr->record.common->keys.channel);
 	}
 
 	return ret;
@@ -843,59 +891,37 @@ ipmi_sensor_get_reading(struct ipmi_intf *intf, int argc, char **argv)
 static int
 ipmi_sensor_get(struct ipmi_intf *intf, int argc, char **argv)
 {
-	struct sdr_record_list *sdr;
 	int i, v;
 	int rc = 0;
+	struct sdr_record_list *sdr;
 
-	if (argc < 1 || strncmp(argv[0], "help", 4) == 0) {
-		lprintf(LOG_NOTICE, "sensor get <id> ... [id]");
-		lprintf(LOG_NOTICE, "   id        : name of desired sensor");
-		return -1;
+	if (argc < 1) {
+		lprintf(LOG_ERR, "Not enough parameters given.");
+		printf_sensor_get_usage();
+		return (-1);
+	} else if (strcmp(argv[0], "help") == 0) {
+		printf_sensor_get_usage();
+		return 0;
 	}
 	printf("Locating sensor record...\n");
-
 	/* lookup by sensor name */
 	for (i = 0; i < argc; i++) {
-		int r = 0;
-
 		sdr = ipmi_sdr_find_sdr_byid(intf, argv[i]);
 		if (sdr == NULL) {
 			lprintf(LOG_ERR, "Sensor data record \"%s\" not found!",
-				argv[i]);
+					argv[i]);
 			rc = -1;
 			continue;
 		}
-
 		/* need to set verbose level to 1 */
 		v = verbose;
 		verbose = 1;
-		switch (sdr->type) {
-		case SDR_RECORD_TYPE_FULL_SENSOR:
-		case SDR_RECORD_TYPE_COMPACT_SENSOR:
-			r = ipmi_sensor_print_fc(intf, sdr->record.common,
-						   sdr->type);
-			break;
-		case SDR_RECORD_TYPE_EVENTONLY_SENSOR:
-			r = ipmi_sdr_print_sensor_eventonly(intf,
-							    sdr->record.
-							    eventonly);
-			break;
-		case SDR_RECORD_TYPE_FRU_DEVICE_LOCATOR:
-			r = ipmi_sdr_print_sensor_fru_locator(intf,
-							      sdr->record.
-							      fruloc);
-			break;
-		case SDR_RECORD_TYPE_MC_DEVICE_LOCATOR:
-			r = ipmi_sdr_print_sensor_mc_locator(intf,
-							     sdr->record.mcloc);
-			break;
+		if (ipmi_sdr_print_listentry(intf, sdr) < 0) {
+			rc = (-1);
 		}
 		verbose = v;
-
-		/* save errors */
-		rc = (r == 0) ? rc : r;
+		sdr = NULL;
 	}
-
 	return rc;
 }
 
@@ -923,3 +949,14 @@ ipmi_sensor_main(struct ipmi_intf *intf, int argc, char **argv)
 
 	return rc;
 }
+
+/* printf_sensor_get_usage - print usage for # ipmitool sensor get NAC;
+ *
+ * @returns: void
+ */
+void
+printf_sensor_get_usage()
+{
+	lprintf(LOG_NOTICE, "sensor get <id> ... [id]");
+	lprintf(LOG_NOTICE, "   id        : name of desired sensor");
+}
