@@ -163,8 +163,7 @@ ipmi_sol_payload_access_status(struct ipmi_intf * intf,
 	rsp = intf->sendrecv(intf, &req);
 
 	if (rsp == NULL) {
-		lprintf(LOG_ERR, "Error: Unexpected data length (%d) received",
-			rsp->data_len);
+		lprintf(LOG_ERR, "Error. No valid response received.");
 		return -1;
 	}
 
@@ -208,7 +207,6 @@ ipmi_get_sol_info(
 	req.msg.data_len = 4;
 	req.msg.data     = data;
 
-
 	/*
 	 * set in progress
 	 */
@@ -219,70 +217,70 @@ ipmi_get_sol_info(
 	data[3] = 0x00;                          /* block selector     */
 
 	rsp = intf->sendrecv(intf, &req);
-
-	if (NULL != rsp) {
-		switch (rsp->ccode) {
-			case 0x00:
-				if (rsp->data_len == 2) {
-					params->set_in_progress = rsp->data[1];
-				} else {
-					lprintf(LOG_ERR, "Error: Unexpected data length (%d) received "
-						   "for SOL parameter '%s'",
-						   rsp->data_len,
-						   val2str(data[1], sol_parameter_vals));
-				}
-				break;
-			case 0x80:
-				lprintf(LOG_ERR, "Info: SOL parameter '%s' not supported",
-				    val2str(data[1], sol_parameter_vals));
-				break;
-			default:
-				lprintf(LOG_ERR, "Error requesting SOL parameter '%s': %s",
-					val2str(data[1], sol_parameter_vals),
-					val2str(rsp->ccode, completion_code_vals));
-				return -1;
-		}
-	} else {
+	if (rsp == NULL) {
 		lprintf(LOG_ERR, "Error: No response requesting SOL parameter '%s'",
-			    val2str(data[1], sol_parameter_vals));
+				val2str(data[1], sol_parameter_vals));
+		return (-1);
+	}
+
+	switch (rsp->ccode) {
+		case 0x00:
+			if (rsp->data_len == 2) {
+				params->set_in_progress = rsp->data[1];
+			} else {
+				lprintf(LOG_ERR, "Error: Unexpected data length (%d) received "
+						"for SOL parameter '%s'",
+						rsp->data_len,
+						val2str(data[1], sol_parameter_vals));
+			}
+			break;
+		case 0x80:
+			lprintf(LOG_ERR, "Info: SOL parameter '%s' not supported",
+					val2str(data[1], sol_parameter_vals));
+			break;
+		default:
+			lprintf(LOG_ERR, "Error requesting SOL parameter '%s': %s",
+				val2str(data[1], sol_parameter_vals),
+				val2str(rsp->ccode, completion_code_vals));
+			return (-1);
 	}
 
 	/*
 	 * SOL enable
 	 */
- 	memset(data, 0, sizeof(data));
+	memset(data, 0, sizeof(data));
 	data[0] = channel;                  /* channel number     */
 	data[1] = SOL_PARAMETER_SOL_ENABLE; /* parameter selector */
 	data[2] = 0x00;                     /* set selector       */
 	data[3] = 0x00;                     /* block selector     */
 
 	rsp = intf->sendrecv(intf, &req);
-
-	if (NULL != rsp) {
-		switch (rsp->ccode) {
-			case 0x00:
-				if (rsp->data_len == 2) {
-					params->enabled = rsp->data[1];
-				} else {
-					lprintf(LOG_ERR, "Error: Unexpected data length (%d) received "
-						   "for SOL parameter '%s'",
-						   rsp->data_len,
-						   val2str(data[1], sol_parameter_vals));
-				}
-				break;
-			case 0x80:
-				lprintf(LOG_ERR, "Info: SOL parameter '%s' not supported",
-				    val2str(data[1], sol_parameter_vals));
-				break;
-			default:
-				lprintf(LOG_ERR, "Error requesting SOL parameter '%s': %s",
-					val2str(data[1], sol_parameter_vals),
-					val2str(rsp->ccode, completion_code_vals));
-				return -1;
-		}
-	} else {
+	if (rsp == NULL) {
 		lprintf(LOG_ERR, "Error: No response requesting SOL parameter '%s'",
-			    val2str(data[1], sol_parameter_vals));
+				val2str(data[1], sol_parameter_vals));
+		return (-1);
+	}
+
+	switch (rsp->ccode) {
+		case 0x00:
+			if (rsp->data_len == 2) {
+				params->enabled = rsp->data[1];
+			} else {
+				lprintf(LOG_ERR, "Error: Unexpected data length (%d) received "
+						"for SOL parameter '%s'",
+						rsp->data_len,
+						val2str(data[1], sol_parameter_vals));
+			}
+			break;
+		case 0x80:
+			lprintf(LOG_ERR, "Info: SOL parameter '%s' not supported",
+					val2str(data[1], sol_parameter_vals));
+			break;
+		default:
+			lprintf(LOG_ERR, "Error requesting SOL parameter '%s': %s",
+				val2str(data[1], sol_parameter_vals),
+				val2str(rsp->ccode, completion_code_vals));
+			return (-1);
 	}
 
 	/*
@@ -295,33 +293,34 @@ ipmi_get_sol_info(
 	data[3] = 0x00;                             /* block selector     */
 
 	rsp = intf->sendrecv(intf, &req);
-	if (NULL != rsp) {
-		switch (rsp->ccode) {
-			case 0x00:
-				if (rsp->data_len == 2) {
-					params->force_encryption     = ((rsp->data[1] & 0x80)? 1 : 0);
-					params->force_authentication = ((rsp->data[1] & 0x40)? 1 : 0);
-					params->privilege_level      = rsp->data[1] & 0x0F;
-				} else {
-					lprintf(LOG_ERR, "Error: Unexpected data length (%d) received "
-						   "for SOL parameter '%s'",
-						   rsp->data_len,
-						   val2str(data[1], sol_parameter_vals));
-				}
-				break;
-			case 0x80:
-				lprintf(LOG_ERR, "Info: SOL parameter '%s' not supported",
-				    val2str(data[1], sol_parameter_vals));
-				break;
-			default:
-				lprintf(LOG_ERR, "Error requesting SOL parameter '%s': %s",
-					val2str(data[1], sol_parameter_vals),
-					val2str(rsp->ccode, completion_code_vals));
-				return -1;
-		}
-	} else {
+	if (rsp == NULL) {
 		lprintf(LOG_ERR, "Error: No response requesting SOL parameter '%s'",
-			    val2str(data[1], sol_parameter_vals));
+				val2str(data[1], sol_parameter_vals));
+		return (-1);
+	}
+
+	switch (rsp->ccode) {
+		case 0x00:
+			if (rsp->data_len == 2) {
+				params->force_encryption     = ((rsp->data[1] & 0x80)? 1 : 0);
+				params->force_authentication = ((rsp->data[1] & 0x40)? 1 : 0);
+				params->privilege_level      = rsp->data[1] & 0x0F;
+			} else {
+				lprintf(LOG_ERR, "Error: Unexpected data length (%d) received "
+						"for SOL parameter '%s'",
+						rsp->data_len,
+						val2str(data[1], sol_parameter_vals));
+			}
+			break;
+		case 0x80:
+			lprintf(LOG_ERR, "Info: SOL parameter '%s' not supported",
+					val2str(data[1], sol_parameter_vals));
+			break;
+		default:
+			lprintf(LOG_ERR, "Error requesting SOL parameter '%s': %s",
+				val2str(data[1], sol_parameter_vals),
+				val2str(rsp->ccode, completion_code_vals));
+			return (-1);
 	}
 
 	/*
@@ -334,32 +333,33 @@ ipmi_get_sol_info(
 	data[3] = 0x00;                             /* block selector     */
 
 	rsp = intf->sendrecv(intf, &req);
-	if (NULL != rsp) {
-		switch (rsp->ccode) {
-			case 0x00:
-				if (rsp->data_len == 3) {
-					params->character_accumulate_level = rsp->data[1];
-					params->character_send_threshold   = rsp->data[2];
-				} else {
-					lprintf(LOG_ERR, "Error: Unexpected data length (%d) received "
-						   "for SOL parameter '%s'",
-						   rsp->data_len,
-						   val2str(data[1], sol_parameter_vals));
-				}
-				break;
-			case 0x80:
-				lprintf(LOG_ERR, "Info: SOL parameter '%s' not supported",
-				    val2str(data[1], sol_parameter_vals));
-				break;
-			default:
-				lprintf(LOG_ERR, "Error requesting SOL parameter '%s': %s",
-					val2str(data[1], sol_parameter_vals),
-					val2str(rsp->ccode, completion_code_vals));
-				return -1;
-		}
-	} else {
+	if (rsp == NULL) {
 		lprintf(LOG_ERR, "Error: No response requesting SOL parameter '%s'",
-			    val2str(data[1], sol_parameter_vals));
+				val2str(data[1], sol_parameter_vals));
+		return (-1);
+	}
+
+	switch (rsp->ccode) {
+		case 0x00:
+			if (rsp->data_len == 3) {
+				params->character_accumulate_level = rsp->data[1];
+				params->character_send_threshold   = rsp->data[2];
+			} else {
+				lprintf(LOG_ERR, "Error: Unexpected data length (%d) received "
+						"for SOL parameter '%s'",
+						rsp->data_len,
+						val2str(data[1], sol_parameter_vals));
+			}
+			break;
+		case 0x80:
+			lprintf(LOG_ERR, "Info: SOL parameter '%s' not supported",
+					val2str(data[1], sol_parameter_vals));
+			break;
+		default:
+			lprintf(LOG_ERR, "Error requesting SOL parameter '%s': %s",
+				val2str(data[1], sol_parameter_vals),
+				val2str(rsp->ccode, completion_code_vals));
+			return (-1);
 	}
 
 	/*
@@ -372,32 +372,33 @@ ipmi_get_sol_info(
 	data[3] = 0x00;                    /* block selector     */
 
 	rsp = intf->sendrecv(intf, &req);
-	if (NULL != rsp) {
-		switch (rsp->ccode) {
-			case 0x00:
-				if (rsp->data_len == 3) {
-					params->retry_count    = rsp->data[1];
-					params->retry_interval = rsp->data[2];
-				} else {
-					lprintf(LOG_ERR, "Error: Unexpected data length (%d) received "
-						   "for SOL parameter '%s'",
-						   rsp->data_len,
-						   val2str(data[1], sol_parameter_vals));
-				}
-				break;
-			case 0x80:
-				lprintf(LOG_ERR, "Info: SOL parameter '%s' not supported",
-				    val2str(data[1], sol_parameter_vals));
-				break;
-			default:
-				lprintf(LOG_ERR, "Error requesting SOL parameter '%s': %s",
-					val2str(data[1], sol_parameter_vals),
-					val2str(rsp->ccode, completion_code_vals));
-				return -1;
-		}
-	} else {
+	if (rsp == NULL) {
 		lprintf(LOG_ERR, "Error: No response requesting SOL parameter '%s'",
-			    val2str(data[1], sol_parameter_vals));
+				val2str(data[1], sol_parameter_vals));
+		return (-1);
+	}
+
+	switch (rsp->ccode) {
+		case 0x00:
+			if (rsp->data_len == 3) {
+				params->retry_count    = rsp->data[1];
+				params->retry_interval = rsp->data[2];
+			} else {
+				lprintf(LOG_ERR, "Error: Unexpected data length (%d) received "
+						"for SOL parameter '%s'",
+						rsp->data_len,
+						val2str(data[1], sol_parameter_vals));
+			}
+			break;
+		case 0x80:
+			lprintf(LOG_ERR, "Info: SOL parameter '%s' not supported",
+					val2str(data[1], sol_parameter_vals));
+			break;
+		default:
+			lprintf(LOG_ERR, "Error requesting SOL parameter '%s': %s",
+				val2str(data[1], sol_parameter_vals),
+				val2str(rsp->ccode, completion_code_vals));
+			return (-1);
 	}
 
 	/*
@@ -405,36 +406,37 @@ ipmi_get_sol_info(
 	 */
 	memset(data, 0, sizeof(data));
 	data[0] = channel;                                 /* channel number     */
- 	data[1] = SOL_PARAMETER_SOL_NON_VOLATILE_BIT_RATE; /* parameter selector */
- 	data[2] = 0x00;                                    /* set selector       */
+	data[1] = SOL_PARAMETER_SOL_NON_VOLATILE_BIT_RATE; /* parameter selector */
+	data[2] = 0x00;                                    /* set selector       */
 	data[3] = 0x00;                                    /* block selector     */
 
 	rsp = intf->sendrecv(intf, &req);
-	if (NULL != rsp) {
-		switch (rsp->ccode) {
-			case 0x00:
-				if (rsp->data_len == 2) {
-					params->non_volatile_bit_rate = rsp->data[1] & 0x0F;
-				} else {
-					lprintf(LOG_ERR, "Error: Unexpected data length (%d) received "
-						   "for SOL parameter '%s'",
-						   rsp->data_len,
-						   val2str(data[1], sol_parameter_vals));
-				}
-				break;
-			case 0x80:
-				lprintf(LOG_ERR, "Info: SOL parameter '%s' not supported",
-				    val2str(data[1], sol_parameter_vals));
-				break;
-			default:
-				lprintf(LOG_ERR, "Error requesting SOL parameter '%s': %s",
-					val2str(data[1], sol_parameter_vals),
-					val2str(rsp->ccode, completion_code_vals));
-				return -1;
-		}
-	} else {
+	if (rsp == NULL) {
 		lprintf(LOG_ERR, "Error: No response requesting SOL parameter '%s'",
-			    val2str(data[1], sol_parameter_vals));
+				val2str(data[1], sol_parameter_vals));
+		return (-1);
+	}
+
+	switch (rsp->ccode) {
+		case 0x00:
+			if (rsp->data_len == 2) {
+				params->non_volatile_bit_rate = rsp->data[1] & 0x0F;
+			} else {
+				lprintf(LOG_ERR, "Error: Unexpected data length (%d) received "
+						"for SOL parameter '%s'",
+						rsp->data_len,
+						val2str(data[1], sol_parameter_vals));
+			}
+			break;
+		case 0x80:
+			lprintf(LOG_ERR, "Info: SOL parameter '%s' not supported",
+					val2str(data[1], sol_parameter_vals));
+			break;
+		default:
+			lprintf(LOG_ERR, "Error requesting SOL parameter '%s': %s",
+				val2str(data[1], sol_parameter_vals),
+				val2str(rsp->ccode, completion_code_vals));
+			return (-1);
 	}
 
 	/*
@@ -447,31 +449,32 @@ ipmi_get_sol_info(
 	data[3] = 0x00;                                /* block selector     */
 
 	rsp = intf->sendrecv(intf, &req);
-	if (NULL != rsp) {
-		switch (rsp->ccode) {
-			case 0x00:
-				if (rsp->data_len == 2) {
-					params->volatile_bit_rate = rsp->data[1] & 0x0F;
-				} else {
-					lprintf(LOG_ERR, "Error: Unexpected data length (%d) received "
-						   "for SOL parameter '%s'",
-						   rsp->data_len,
-						   val2str(data[1], sol_parameter_vals));
-				}
-				break;
-			case 0x80:
-				lprintf(LOG_ERR, "Info: SOL parameter '%s' not supported",
-				    val2str(data[1], sol_parameter_vals));
-				break;
-			default:
-				lprintf(LOG_ERR, "Error requesting SOL parameter '%s': %s",
-					val2str(data[1], sol_parameter_vals),
-					val2str(rsp->ccode, completion_code_vals));
-				return -1;
-		}
-	} else {
+	if (rsp == NULL) {
 		lprintf(LOG_ERR, "Error: No response requesting SOL parameter '%s'",
-			    val2str(data[1], sol_parameter_vals));
+				val2str(data[1], sol_parameter_vals));
+		return (-1);
+	}
+
+	switch (rsp->ccode) {
+		case 0x00:
+			if (rsp->data_len == 2) {
+				params->volatile_bit_rate = rsp->data[1] & 0x0F;
+			} else {
+				lprintf(LOG_ERR, "Error: Unexpected data length (%d) received "
+						"for SOL parameter '%s'",
+						rsp->data_len,
+						val2str(data[1], sol_parameter_vals));
+			}
+			break;
+		case 0x80:
+			lprintf(LOG_ERR, "Info: SOL parameter '%s' not supported",
+					val2str(data[1], sol_parameter_vals));
+			break;
+		default:
+			lprintf(LOG_ERR, "Error requesting SOL parameter '%s': %s",
+				val2str(data[1], sol_parameter_vals),
+				val2str(rsp->ccode, completion_code_vals));
+			return (-1);
 	}
 
 	/*
@@ -484,32 +487,33 @@ ipmi_get_sol_info(
 	data[3] = 0x00;                              /* block selector     */
 
 	rsp = intf->sendrecv(intf, &req);
-	if (NULL != rsp) {
-		switch (rsp->ccode) {
-			case 0x00:
-				if (rsp->data_len == 2) {
-					params->payload_channel = rsp->data[1];
-				} else {
-					lprintf(LOG_ERR, "Error: Unexpected data length (%d) received "
-						   "for SOL parameter '%s'",
-						   rsp->data_len,
-						   val2str(data[1], sol_parameter_vals));
-				}
-				break;
-			case 0x80:
-				lprintf(LOG_ERR, "Info: SOL parameter '%s' not supported - defaulting to 0x%02x",
-				    val2str(data[1], sol_parameter_vals), channel);
-				params->payload_channel = channel;
-				break;
-			default:
-				lprintf(LOG_ERR, "Error requesting SOL parameter '%s': %s",
+	if (rsp == NULL) {
+		lprintf(LOG_ERR, "Error: No response requesting SOL parameter '%s'",
+				val2str(data[1], sol_parameter_vals));
+		return (-1);
+	}
+
+	switch (rsp->ccode) {
+		case 0x00:
+			if (rsp->data_len == 2) {
+				params->payload_channel = rsp->data[1];
+			} else {
+				lprintf(LOG_ERR, "Error: Unexpected data length (%d) received "
+						"for SOL parameter '%s'",
+						rsp->data_len,
+						val2str(data[1], sol_parameter_vals));
+			}
+			break;
+		case 0x80:
+			lprintf(LOG_ERR, "Info: SOL parameter '%s' not supported - defaulting to 0x%02x",
+					val2str(data[1], sol_parameter_vals), channel);
+			params->payload_channel = channel;
+			break;
+		default:
+			lprintf(LOG_ERR, "Error requesting SOL parameter '%s': %s",
 					val2str(data[1], sol_parameter_vals),
 					val2str(rsp->ccode, completion_code_vals));
-				return -1;
-		}
-	} else {
-		lprintf(LOG_ERR, "Error: No response requesting SOL parameter '%s'",
-			    val2str(data[1], sol_parameter_vals));
+			return (-1);
 	}
 
 	/*
@@ -522,36 +526,41 @@ ipmi_get_sol_info(
 	data[3] = 0x00;                           /* block selector     */
 
 	rsp = intf->sendrecv(intf, &req);
-	if (NULL != rsp) {
-		switch (rsp->ccode) {
-			case 0x00:
-				if (rsp->data_len == 3) {
-					params->payload_port = (rsp->data[1]) |	(rsp->data[2] << 8);
-				} else {
-					lprintf(LOG_ERR, "Error: Unexpected data length (%d) received "
-						   "for SOL parameter '%s'",
-						   rsp->data_len,
-						   val2str(data[1], sol_parameter_vals));
-				}
-				break;
-			case 0x80:
-            if( intf->session != NULL ){
-               lprintf(LOG_ERR, "Info: SOL parameter '%s' not supported - defaulting to %d", val2str(data[1], sol_parameter_vals), intf->session->port);
-               params->payload_port = intf->session->port;
-            } else {
-               lprintf(LOG_ERR, "Info: SOL parameter '%s' not supported - can't determine which payload port to use on NULL session" );
-               return -1;               
-            }
-				break;
-			default:
-				lprintf(LOG_ERR, "Error requesting SOL parameter '%s': %s",
-					val2str(data[1], sol_parameter_vals),
-					val2str(rsp->ccode, completion_code_vals));
-				return -1;
-		}
-	} else {
+	if (rsp == NULL) {
 		lprintf(LOG_ERR, "Error: No response requesting SOL parameter '%s'",
-			    val2str(data[1], sol_parameter_vals));
+				val2str(data[1], sol_parameter_vals));
+		return (-1);
+	}
+
+	switch (rsp->ccode) {
+		case 0x00:
+			if (rsp->data_len == 3) {
+				params->payload_port = (rsp->data[1]) | (rsp->data[2] << 8);
+			} else {
+				lprintf(LOG_ERR, "Error: Unexpected data length (%d) received "
+						"for SOL parameter '%s'",
+						rsp->data_len,
+						val2str(data[1], sol_parameter_vals));
+			}
+			break;
+		case 0x80:
+			if( intf->session != NULL ) {
+				lprintf(LOG_ERR, "Info: SOL parameter '%s' not supported - defaulting to %d",
+						val2str(data[1], sol_parameter_vals), intf->session->port);
+				params->payload_port = intf->session->port;
+			} else {
+				lprintf(LOG_ERR,
+						"Info: SOL parameter '%s' not supported - can't determine which "
+						"payload port to use on NULL session",
+						val2str(data[1], sol_parameter_vals));
+						return (-1);               
+			}
+			break;
+		default:
+			lprintf(LOG_ERR, "Error requesting SOL parameter '%s': %s",
+				val2str(data[1], sol_parameter_vals),
+				val2str(rsp->ccode, completion_code_vals));
+			return (-1);
 	}
 
 	return 0;
@@ -650,21 +659,14 @@ int ipmi_sol_set_param_isvalid_uint8_t( const char *strval,
 					uint8_t maxval,
 					uint8_t *out_value)
 {
-	char *end;
-	long val = strtol(strval, &end, base);
-
-	if ((val < minval)
-			|| (val > maxval)
-			|| (*end != '\0')) {
+	if (str2uchar(strval, out_value) != 0 || (*out_value < minval)
+			|| (*out_value > maxval)) {
 		lprintf(LOG_ERR, "Invalid value %s for parameter %s",
 			strval, name);
 		lprintf(LOG_ERR, "Valid values are %d-%d", minval, maxval);
 		return -1;
 	}
-	else {
-		*out_value = val;
-		return 0;
-	}
+	return 0;
 }
 
 
@@ -1283,12 +1285,17 @@ output(struct ipmi_rs * rsp)
  * ipmi_sol_deactivate
  */
 static int
-ipmi_sol_deactivate(struct ipmi_intf * intf)
+ipmi_sol_deactivate(struct ipmi_intf * intf, int instance)
 {
 	struct ipmi_rs * rsp;
 	struct ipmi_rq   req;
 	uint8_t          data[6];
 
+	if ((instance <= 0) || (instance > 15)) {
+		lprintf(LOG_ERR, "Error: Instance must range from 1 to 15");
+		return -1;
+	}
+
 	memset(&req, 0, sizeof(req));
 	req.msg.netfn    = IPMI_NETFN_APP;
 	req.msg.cmd      = IPMI_DEACTIVATE_PAYLOAD;
@@ -1296,8 +1303,8 @@ ipmi_sol_deactivate(struct ipmi_intf * intf)
 	req.msg.data     = data;
 
 	bzero(data, sizeof(data));
-	data[0] = IPMI_PAYLOAD_TYPE_SOL;  /* payload type              */
-	data[1] = 1;                      /* payload instance.  Guess! */
+	data[0] = IPMI_PAYLOAD_TYPE_SOL;  /* payload type      */
+	data[1] = instance;               /* payload instance. */
 
 	/* Lots of important data */
 	data[2] = 0;
@@ -1524,7 +1531,7 @@ ipmi_sol_keepalive_using_getdeviceid(struct ipmi_intf * intf)
  * ipmi_sol_red_pill
  */
 static int
-ipmi_sol_red_pill(struct ipmi_intf * intf)
+ipmi_sol_red_pill(struct ipmi_intf * intf, int instance)
 {
 	char   * buffer;
 	int    numRead;
@@ -1537,6 +1544,10 @@ ipmi_sol_red_pill(struct ipmi_intf * intf)
 	int    keepAliveRet = 0;
 	int    retrySol = 0;
 
+	/* Subtract SOL header from max_inbound_payload_size */
+	if (buffer_size > 4)
+		buffer_size -= 4;
+
 	buffer = (char*)malloc(buffer_size);
 	if (buffer == NULL) {
 		lprintf(LOG_ERR, "ipmitool: malloc failure"); 
@@ -1554,43 +1565,45 @@ ipmi_sol_red_pill(struct ipmi_intf * intf)
 		FD_SET(0, &read_fds);
 		FD_SET(intf->fd, &read_fds);
 
-		/* Send periodic keepalive packet */
-		if(_use_sol_for_keepalive == 0)
+		if (!ipmi_oem_active(intf,"i82571spt"))
 		{
-			keepAliveRet = ipmi_sol_keepalive_using_getdeviceid(intf);
-		}
-		else
-		{
-			keepAliveRet = ipmi_sol_keepalive_using_sol(intf);
-		}
-		
-		if (keepAliveRet != 0)
-		{
-			/*
-			 * Retrying the keep Alive before declaring a communication
-			 * lost state with the IPMC. Helpful when the payload is 
-			 * reset and brings down the connection temporarily. Otherwise,
-			 * if we send getDevice Id to check the status of IPMC during
-			 * this down time when the connection is restarting, SOL will 
-			 * exit even though the IPMC is available and the session is open.
-			 */
-			if (retrySol == MAX_SOL_RETRY)
+			/* Send periodic keepalive packet */
+			if(_use_sol_for_keepalive == 0)
 			{
-				/* no response to Get Device ID keepalive message */
-				bShouldExit = 1;
-				continue;
+				keepAliveRet = ipmi_sol_keepalive_using_getdeviceid(intf);
 			}
-			else 
-			{ 
-				retrySol++;         
+			else
+			{
+				keepAliveRet = ipmi_sol_keepalive_using_sol(intf);
 			}
-		}
-		else
-		{
-			/* if the keep Alive is successful reset retries to zero */
-			retrySol = 0;
-		}
-
+		
+			if (keepAliveRet != 0)
+			{
+				/*
+				 * Retrying the keep Alive before declaring a communication
+				 * lost state with the IPMC. Helpful when the payload is
+				 * reset and brings down the connection temporarily. Otherwise,
+				 * if we send getDevice Id to check the status of IPMC during
+				 * this down time when the connection is restarting, SOL will
+				 * exit even though the IPMC is available and the session is open.
+				 */
+				if (retrySol == MAX_SOL_RETRY)
+				{
+					/* no response to Get Device ID keepalive message */
+					bShouldExit = 1;
+					continue;
+				}
+				else
+				{
+					retrySol++;
+				}
+			}
+			else
+			{
+				/* if the keep Alive is successful reset retries to zero */
+				retrySol = 0;
+			}
+		} /* !oem="i82571spt" */
 		/* Wait up to half a second */
 		tv.tv_sec =  0;
 		tv.tv_usec = 500000;
@@ -1612,7 +1625,7 @@ ipmi_sol_red_pill(struct ipmi_intf * intf)
 			 */
 			if (FD_ISSET(0, &read_fds))
 	 		{
-				bzero(buffer, sizeof(buffer));
+				memset(buffer, 0, buffer_size);
 				numRead = read(fileno(stdin),
 							   buffer,
 							   buffer_size);
@@ -1670,7 +1683,7 @@ ipmi_sol_red_pill(struct ipmi_intf * intf)
 	{
 		lprintf(LOG_ERR, "Error: No response to keepalive - Terminating session");
 		/* attempt to clean up anyway */
-		ipmi_sol_deactivate(intf);
+		ipmi_sol_deactivate(intf, instance);
 		exit(1);
 	}
 
@@ -1680,7 +1693,7 @@ ipmi_sol_red_pill(struct ipmi_intf * intf)
 		exit(1);
 	}
 	else
-		ipmi_sol_deactivate(intf);
+		ipmi_sol_deactivate(intf, instance);
 
 	return 0;
 }
@@ -1692,7 +1705,8 @@ ipmi_sol_red_pill(struct ipmi_intf * intf)
  * ipmi_sol_activate
  */
 static int
-ipmi_sol_activate(struct ipmi_intf * intf, int looptest, int interval)
+ipmi_sol_activate(struct ipmi_intf * intf, int looptest, int interval,
+		int instance)
 {
 	struct ipmi_rs * rsp;
 	struct ipmi_rq   req;
@@ -1712,6 +1726,11 @@ ipmi_sol_activate(struct ipmi_intf * intf, int looptest, int interval)
 		return -1;
 	}
 
+	if ((instance <= 0) || (instance > 15)) {
+		lprintf(LOG_ERR, "Error: Instance must range from 1 to 15");
+		return -1;
+	}
+
 
 	/*
 	 * Setup a callback so that the lanplus processing knows what
@@ -1728,7 +1747,7 @@ ipmi_sol_activate(struct ipmi_intf * intf, int looptest, int interval)
 	req.msg.data     = data;
 
 	data[0] = IPMI_PAYLOAD_TYPE_SOL;  /* payload type     */
-	data[1] = 1;                      /* payload instance */
+	data[1] = instance;               /* payload instance */
 
 	/* Lots of important data.  Most is default */
 	data[2]  = bSolEncryption?     0x80 : 0;
@@ -1737,6 +1756,20 @@ ipmi_sol_activate(struct ipmi_intf * intf, int looptest, int interval)
 
 	if (ipmi_oem_active(intf, "intelplus")) {
 		data[2] |= IPMI_SOL_BMC_ASSERTS_CTS_MASK_TRUE;
+	} else if (ipmi_oem_active(intf, "i82571spt")) {
+		/*
+		 * A quote from Intel: "Engineering believes the problem
+		 * lies within the Auxiliary data being sent with the
+		 * 'Activate Payload' command from IPMITool.  IPMITool
+		 * sends a C6h which sets some bits having to do with
+		 * encryption and some behavior dealing with CTS DCD/DSR.
+		 * I recommend that the customer modify this request
+		 * to send 08h instead. This is what our internal utility
+		 * sends and it works without issue. I will work with
+		 * engineering to ensure the settings that IPMITool uses
+		 * (C6h) are supported in the future.
+		 */
+		data[2] = 0x08;
 	} else {
 		data[2] |= IPMI_SOL_BMC_ASSERTS_CTS_MASK_FALSE;
 	}
@@ -1834,7 +1867,7 @@ ipmi_sol_activate(struct ipmi_intf * intf, int looptest, int interval)
 
 	if(looptest == 1)
 	{
-		ipmi_sol_deactivate(intf);
+		ipmi_sol_deactivate(intf, instance);
 		usleep(interval*1000);
 		return 0;
 	}
@@ -1845,7 +1878,7 @@ ipmi_sol_activate(struct ipmi_intf * intf, int looptest, int interval)
 	 * 1) STDIN for user input
 	 * 2) The FD for incoming SOL packets
 	 */
-	if (ipmi_sol_red_pill(intf))
+	if (ipmi_sol_red_pill(intf, instance))
 	{
 		lprintf(LOG_ERR, "Error in SOL session");
 		return -1;
@@ -1865,9 +1898,9 @@ print_sol_usage(void)
 	lprintf(LOG_NOTICE, "SOL Commands: info [<channel number>]");
 	lprintf(LOG_NOTICE, "              set <parameter> <value> [channel]");
 	lprintf(LOG_NOTICE, "              payload <enable|disable|status> [channel] [userid]");
-	lprintf(LOG_NOTICE, "              activate [<usesolkeepalive|nokeepalive>]");
-	lprintf(LOG_NOTICE, "              deactivate");
-	lprintf(LOG_NOTICE, "              looptest [<loop times>] [<loop interval(in ms)>]");
+	lprintf(LOG_NOTICE, "              activate [<usesolkeepalive|nokeepalive>] [instance=<number>]");
+	lprintf(LOG_NOTICE, "              deactivate [instance=<number>]");
+	lprintf(LOG_NOTICE, "              looptest [<loop times> [<loop interval(in ms)> [<instance>]]]");
 }
 
 
@@ -1920,10 +1953,11 @@ ipmi_sol_main(struct ipmi_intf * intf, int argc, char ** argv)
 
 		if (argc == 1)
 			channel = 0x0E; /* Ask about the current channel */
-		else if (argc == 2)
-			channel = (uint8_t)strtol(argv[1], NULL, 0);
-		else
-		{
+		else if (argc == 2) {
+			if (is_ipmi_channel_num(argv[1], &channel) != 0) {
+				return (-1);
+			}
+		} else {
 			print_sol_usage();
 			return -1;
 		}
@@ -1951,13 +1985,16 @@ ipmi_sol_main(struct ipmi_intf * intf, int argc, char ** argv)
 			return -1;
 		}
 
-		if (argc >= 3)
-		{
-			channel = (uint8_t)strtol(argv[2], NULL, 0);
+		if (argc >= 3) {
+			if (is_ipmi_channel_num(argv[2], &channel) != 0) {
+				return (-1);
+			}
 		}
 		if (argc == 4)
 		{
-			userid = (uint8_t)strtol(argv[3], NULL, 0);
+			if (is_ipmi_user_id(argv[3], &userid) != 0) {
+				return (-1);
+			}
 		}
 
 		if (!strncmp(argv[1], "enable", 6))
@@ -1997,12 +2034,17 @@ ipmi_sol_main(struct ipmi_intf * intf, int argc, char ** argv)
 		{
 			if (!strncmp(argv[3], "noguard", 7))
 				guard = 0;
-			else
-				channel = (uint8_t)strtol(argv[3], NULL, 0);
+			else {
+				if (is_ipmi_channel_num(argv[3], &channel) != 0) {
+					return (-1);
+				}
+			}
 		}
 		else if (argc == 5)
 		{
-			channel = (uint8_t)strtol(argv[3], NULL, 0);
+			if (is_ipmi_channel_num(argv[3], &channel) != 0) {
+				return (-1);
+			}
 			if (!strncmp(argv[4], "noguard", 7))
 				guard = 0;
 		}
@@ -2020,32 +2062,50 @@ ipmi_sol_main(struct ipmi_intf * intf, int argc, char ** argv)
 	 * Activate
 	 */
  	else if (!strncmp(argv[0], "activate", 8)) {
+		int i;
+		uint8_t instance = 1;
 
-		if (argc > 2) {
-			print_sol_usage();
-			return -1;
-		}
-
-		if (argc == 2) {
-			if (!strncmp(argv[1], "usesolkeepalive", 15))
+		for (i = 1; i < argc; i++) {
+			if (!strncmp(argv[i], "usesolkeepalive", 15)) {
 				_use_sol_for_keepalive = 1;
-			else if (!strncmp(argv[1], "nokeepalive", 11))
+			} else if (!strncmp(argv[i], "nokeepalive", 11)) {
 				_disable_keepalive = 1;
-			else {
+			} else if (!strncmp(argv[i], "instance=", 9)) {
+				if (str2uchar(argv[i] + 9, &instance) != 0) {
+					lprintf(LOG_ERR, "Given instance '%s' is invalid.", argv[i] + 9);
+					print_sol_usage();
+					return -1;
+				}
+			} else {
 				print_sol_usage();
 				return -1;
 			}
 		}
-		retval = ipmi_sol_activate(intf, 0, 0);
+		retval = ipmi_sol_activate(intf, 0, 0, instance);
 	}
 
 
 	/*
 	 * Dectivate
 	 */
-	else if (!strncmp(argv[0], "deactivate", 10))
-		retval = ipmi_sol_deactivate(intf);
+	else if (!strncmp(argv[0], "deactivate", 10)) {
+		int i;
+		uint8_t instance = 1;
 
+		for (i = 1; i < argc; i++) {
+			if (!strncmp(argv[i], "instance=", 9)) {
+				if (str2uchar(argv[i] + 9, &instance) != 0) {
+					lprintf(LOG_ERR, "Given instance '%s' is invalid.", argv[i] + 9);
+					print_sol_usage();
+					return -1;
+				}
+			} else {
+				print_sol_usage();
+				return -1;
+			}
+		}
+		retval = ipmi_sol_deactivate(intf, instance);
+	}
 
 	/*
 	 * SOL loop test: Activate and then Dectivate
@@ -2054,27 +2114,43 @@ ipmi_sol_main(struct ipmi_intf * intf, int argc, char ** argv)
 	{
 		int cnt = 200;
 		int interval = 100; /* Unit is: ms */
+		uint8_t instance;
 
-		if (argc > 3)
+		if (argc > 4)
 		{
 			print_sol_usage();
 			return -1;
 		}
 		if (argc != 1) /* at least 2 */
 		{
-			cnt = strtol(argv[1], NULL, 10);
+			if (str2int(argv[1], &cnt) != 0) {
+				lprintf(LOG_ERR, "Given cnt '%s' is invalid.",
+						argv[1]);
+				return (-1);
+			}
 			if(cnt <= 0) cnt = 200;
 		}
-		if (argc == 3)
+		if (argc >= 3)
 		{
-			interval = strtol(argv[2], NULL, 10);
+			if (str2int(argv[2], &interval) != 0) {
+				lprintf(LOG_ERR, "Given interval '%s' is invalid.",
+						argv[2]);
+				return (-1);
+			}
 			if(interval < 0) interval = 0;
 		}
+		if (argc >= 4) {
+			if (str2uchar(argv[3], &instance) != 0) {
+				lprintf(LOG_ERR, "Given instance '%s' is invalid.", argv[3]);
+				print_sol_usage();
+				return -1;
+			}
+		}
 
 		while (cnt > 0)
 		{
 			printf("remain loop test counter: %d\n", cnt);
-			retval = ipmi_sol_activate(intf, 1, interval);
+			retval = ipmi_sol_activate(intf, 1, interval, instance);
 			if (retval)
 			{
 				printf("SOL looptest failed: %d\n", retval);
