@@ -67,6 +67,7 @@ const struct valstr ipmi_oem_info[] = {
    { IPMI_OEM_NOKIA_SIEMENS_NETWORKS, "Nokia Siemens Networks" },
    { IPMI_OEM_BULL,                   "Bull Company" },
    { IPMI_OEM_PPS,                    "Pigeon Point Systems" },
+   { IPMI_OEM_BROADCOM,               "Broadcom Corporation" },
    { 0xffff , NULL },
 };
 
@@ -112,6 +113,8 @@ const struct oemvalstr ipmi_oem_product_info[] = {
    { IPMI_OEM_KONTRON,5007, "AT8050" },
    { IPMI_OEM_KONTRON,5301, "AT8400" },
    { IPMI_OEM_KONTRON,5303, "AT8901" },
+   /* Broadcom */
+   { IPMI_OEM_BROADCOM, 5725, "BCM5725" },
 
    { 0xffffff        , 0xffff , NULL },
  };
@@ -260,26 +263,40 @@ const struct valstr entity_id_vals[] = {
 	{ 0x26, "Remote Management Device" },
 	{ 0x27, "External Environment" },
 	{ 0x28, "Battery" },
-   { 0x29, "Processing Blade" },
-   { 0x2A, "Connectivity Switch" },
-   { 0x2B, "Processor/Memory Module" },
-   { 0x2C, "I/O Module" },
-   { 0x2D, "Processor/IO Module" },
-   { 0x2E, "Management Controller Firmware" },
-   { 0x2F, "IPMI Channel" },
-   { 0x30, "PCI Bus" },
-   { 0x31, "PCI Express Bus" },
-   { 0x32, "SCSI Bus (parallel)" },
-   { 0x33, "SATA/SAS Bus" },
-   { 0x34, "Processor/Front-Side Bus" },
-   /* PICMG */
-   { 0xA0, "PICMG Front Board" },
-   { 0xC0, "PICMG Rear Transition Module" },
-   { 0xC1, "PICMG AdvancedMC Module" },
-   { 0xF0, "PICMG Shelf Management Controller" },
-   { 0xF1, "PICMG Filtration Unit" },
-   { 0xF2, "PICMG Shelf FRU Information" },
-   { 0xF3, "PICMG Alarm Panel" },
+	{ 0x29, "Processing Blade" },
+	{ 0x2A, "Connectivity Switch" },
+	{ 0x2B, "Processor/Memory Module" },
+	{ 0x2C, "I/O Module" },
+	{ 0x2D, "Processor/IO Module" },
+	{ 0x2E, "Management Controller Firmware" },
+	{ 0x2F, "IPMI Channel" },
+	{ 0x30, "PCI Bus" },
+	{ 0x31, "PCI Express Bus" },
+	{ 0x32, "SCSI Bus (parallel)" },
+	{ 0x33, "SATA/SAS Bus" },
+	{ 0x34, "Processor/Front-Side Bus" },
+	{ 0x35, "Real Time Clock(RTC)" },
+	{ 0x36, "Reserved" },
+	{ 0x37, "Air Inlet" },
+	{ 0x38, "Reserved" },
+	{ 0x39, "Reserved" },
+	{ 0x3A, "Reserved" },
+	{ 0x3B, "Reserved" },
+	{ 0x3C, "Reserved" },
+	{ 0x3D, "Reserved" },
+	{ 0x3E, "Reserved" },
+	{ 0x3F, "Reserved" },
+	{ 0x40, "Air Inlet" },
+	{ 0x41, "Processor" },
+	{ 0x42, "Baseboard/Main System Board" },
+	/* PICMG */
+	{ 0xA0, "PICMG Front Board" },
+	{ 0xC0, "PICMG Rear Transition Module" },
+	{ 0xC1, "PICMG AdvancedMC Module" },
+	{ 0xF0, "PICMG Shelf Management Controller" },
+	{ 0xF1, "PICMG Filtration Unit" },
+	{ 0xF2, "PICMG Shelf FRU Information" },
+	{ 0xF3, "PICMG Alarm Panel" },
 	{ 0x00, NULL },
 };
 
