@@ -437,6 +437,13 @@ ipmi_tsol_main(struct ipmi_intf * intf, int argc, char ** argv)
 				intf->session->hostname);
 			return -1;
 		}
+		if (host->h_addrtype != AF_INET) {
+			lprintf(LOG_ERR,
+					"Address lookup for %s failed. Got %s, expected IPv4 address.",
+					intf->session->hostname,
+					(host->h_addrtype == AF_INET6) ? "IPv6" : "Unknown");
+			return (-1);
+		}
 		intf->session->addr.sin_family = host->h_addrtype;
 		memcpy(&intf->session->addr.sin_addr, host->h_addr, host->h_length);
 	}
@@ -446,25 +453,32 @@ ipmi_tsol_main(struct ipmi_intf * intf, int argc, char ** argv)
 		lprintf(LOG_ERR, "Can't open port %d", port);
 		return -1;
 	}
-	bind(fd_socket, (struct sockaddr *)&sin, sizeof(sin));
+	if (-1 == bind(fd_socket, (struct sockaddr *)&sin, sizeof(sin))) {
+		lprintf(LOG_ERR, "Failed to bind socket.");
+		return -1;
+	}
 
 	/*
 	 * retrieve local IP address if not supplied on command line
 	 */
 	if (recvip == NULL) {
 		result = intf->open(intf);	/* must connect first */
-		if (result < 0)
+		if (result < 0) {
+			close(fd_socket);
 			return -1;
+		}
 
 		mylen = sizeof(myaddr);
 		if (getsockname(intf->fd, (struct sockaddr *)&myaddr, &mylen) < 0) {
 			lperror(LOG_ERR, "getsockname failed");
+			close(fd_socket);
 			return -1;
 		}
 
 		recvip = inet_ntoa(myaddr.sin_addr);
 		if (recvip == NULL) {
 			lprintf(LOG_ERR, "Unable to find local IP address");
+			close(fd_socket);
 			return -1;
 		}
 	}
@@ -482,6 +496,7 @@ ipmi_tsol_main(struct ipmi_intf * intf, int argc, char ** argv)
 	result = ipmi_tsol_start(intf, recvip, port);
         if (result < 0) {
 		lprintf(LOG_ERR, "Error starting SOL");
+		close(fd_socket);
                 return -1;
         }
 
