@@ -173,9 +173,12 @@ ipmi_req_remove_entry(uint8_t seq, uint8_t cmd)
 			else
 				ipmi_req_entries_tail = NULL;
 		}
-		if (e->msg_data)
+		if (e->msg_data) {
 			free(e->msg_data);
+			e->msg_data = NULL;
+		}
 		free(e);
+		e = NULL;
 	}
 }
 
@@ -194,6 +197,7 @@ ipmi_req_clear_entries(void)
 			e = p;
 		} else {
 			free(e);
+			e = NULL;
 			break;
 		}
 	}
@@ -206,8 +210,12 @@ get_random(void *data, int len)
 	int fd = open("/dev/urandom", O_RDONLY);
 	int rv;
 
-	if (fd < 0 || len < 0)
+	if (fd < 0)
 		return errno;
+	if (len < 0) {
+		close(fd);
+		return errno; /* XXX: ORLY? */
+	}
 
 	rv = read(fd, data, len);
 
@@ -390,6 +398,7 @@ ipmi_lan_ping(struct ipmi_intf * intf)
 	rv = ipmi_lan_send_packet(intf, data, len);
 
 	free(data);
+	data = NULL;
 
 	if (rv < 0) {
 		lprintf(LOG_ERR, "Unable to send IPMI presence ping packet");
@@ -703,8 +712,10 @@ ipmi_lan_build_cmd(struct ipmi_intf * intf, struct ipmi_rq * req, int isRetry)
 		// No need to add once again and we will re-use the existing node.
 		// Only thing we have to do is clear the msg_data as we create
 		// a new one below in the code for it.
-		if (entry->msg_data)
+		if (entry->msg_data) {
 			free(entry->msg_data);
+			entry->msg_data = NULL;
+		}
 	}
 	else
 	{
@@ -1064,8 +1075,10 @@ ipmi_lan_send_rsp(struct ipmi_intf * intf, struct ipmi_rs * rsp)
 	msg = ipmi_lan_build_rsp(intf, rsp, &len);
 	if (len <= 0 || msg == NULL) {
 		lprintf(LOG_ERR, "Invalid response packet");
-		if (msg != NULL)
+		if (msg != NULL) {
 			free(msg);
+			msg = NULL;
+		}
 		return -1;
 	}
 
@@ -1074,13 +1087,17 @@ ipmi_lan_send_rsp(struct ipmi_intf * intf, struct ipmi_rs * rsp)
 		    intf->session->addrlen);
 	if (rv < 0) {
 		lprintf(LOG_ERR, "Packet send failed");
-		if (msg != NULL)
+		if (msg != NULL) {
 			free(msg);
+			msg = NULL;
+		}
 		return -1;
 	}
 
-	if (msg != NULL)
+	if (msg != NULL) {
 		free(msg);
+		msg = NULL;
+	}
 	return 0;
 }
 
@@ -1230,8 +1247,10 @@ ipmi_lan_send_sol_payload(struct ipmi_intf * intf,
 	msg = ipmi_lan_build_sol_msg(intf, payload, &len);
 	if (len <= 0 || msg == NULL) {
 		lprintf(LOG_ERR, "Invalid SOL payload packet");
-		if (msg != NULL)
+		if (msg != NULL) {
 			free(msg);
+			msg = NULL;
+		}
 		return NULL;
 	}
 
@@ -1276,6 +1295,10 @@ ipmi_lan_send_sol_payload(struct ipmi_intf * intf,
 		}
 	}
 
+	if (msg != NULL) {
+		free(msg);
+		msg = NULL;
+	}
 	return rsp;
 }
 
@@ -1936,10 +1959,7 @@ ipmi_lan_activate_session(struct ipmi_intf * intf)
 
 	rc = ipmi_get_auth_capabilities_cmd(intf);
 	if (rc < 0) {
-		sleep(1);
-		rc = ipmi_get_auth_capabilities_cmd(intf);
-		if (rc < 0)
-			goto fail;
+		goto fail;
 	}
 
 	rc = ipmi_get_session_challenge_cmd(intf);
@@ -2025,6 +2045,13 @@ ipmi_lan_open(struct ipmi_intf * intf)
 				s->hostname);
 			return -1;
 		}
+		if (host->h_addrtype != AF_INET) {
+			lprintf(LOG_ERR,
+					"Address lookup for %s failed. Got %s, expected IPv4 address.",
+					s->hostname,
+					(host->h_addrtype == AF_INET6) ? "IPv6" : "Unknown");
+			return (-1);
+		}
 		s->addr.sin_family = host->h_addrtype;
 		memcpy(&s->addr.sin_addr, host->h_addr, host->h_length);
 	}
