@@ -349,9 +349,12 @@ ipmi_req_remove_entry(uint8_t seq, uint8_t cmd)
 				ipmi_req_entries_tail = NULL;
 		}
 
-		if (e->msg_data)
+		if (e->msg_data) {
 			free(e->msg_data);
+			e->msg_data = NULL;
+		}
 		free(e);
+		e = NULL;
 	}
 }
 
@@ -564,6 +567,7 @@ ipmiv2_lan_ping(struct ipmi_intf * intf)
 	rv = ipmi_lan_send_packet(intf, data, len);
 
 	free(data);
+	data = NULL;
 
 	if (rv < 0) {
 		lprintf(LOG_ERR, "Unable to send IPMI presence ping packet");
@@ -1352,9 +1356,9 @@ void getIpmiPayloadWireRep(
 	len = 0;
 
 	/* IPMI Message Header -- Figure 13-4 of the IPMI v2.0 spec */
-	if ((intf->target_addr == ourAddress) || (!bridgePossible))
+	if ((intf->target_addr == ourAddress) || (!bridgePossible)) {
 		cs = len;
-	else {
+	} else {
 		bridgedRequest = 1;
 
 		if(intf->transit_addr != ourAddress && intf->transit_addr != 0)
@@ -1405,6 +1409,13 @@ void getIpmiPayloadWireRep(
 		}
 	}
 
+        lprintf(LOG_DEBUG,"%s RqAddr %#x transit %#x:%#x target %#x:%#x "
+		"bridgePossible %d",
+		bridgedRequest ? "Bridging" : "Local",
+		intf->my_addr, intf->transit_addr, intf->transit_channel,
+		intf->target_addr, intf->target_channel,
+		bridgePossible);
+
 	/* rsAddr */
 	msg[len++] = intf->target_addr; /* IPMI_BMC_SLAVE_ADDR; */
 
@@ -1707,6 +1718,7 @@ ipmi_lanplus_build_v2x_msg(
 		lprintf(LOG_ERR, "unsupported payload type 0x%x",
 			payload->payload_type);
 		free(msg);
+		msg = NULL;
 		assert(0);
 		break;
 	}
@@ -2090,7 +2102,7 @@ ipmi_lanplus_send_payload(
 	int                   try = 0;
 	int                   xmit = 1;
 	time_t                ltime;
-	uint32_t	      timeout;
+	uint32_t	      saved_timeout;
 
 	if (!intf->opened && intf->open && intf->open(intf) < 0)
 		return NULL;
@@ -2099,7 +2111,7 @@ ipmi_lanplus_send_payload(
 	 * The session timeout is initialized in the above interface open,
 	 * so it will only be valid after the open completes.
 	 */
-	timeout = session->timeout;
+	saved_timeout = session->timeout;
 	while (try < session->retry) {
 		//ltime = time(NULL);
 
@@ -2295,17 +2307,18 @@ ipmi_lanplus_send_payload(
 		}
 
 		/* only timeout if time exceeds the timeout value */
-		xmit = ((time(NULL) - ltime) > timeout);
+		xmit = ((time(NULL) - ltime) >= session->timeout);
 
 		usleep(5000);
 
 		if (xmit) {
 			/* increment session timeout by 1 second each retry */
-			timeout++;
+			session->timeout++;
 		}
 
 		try++;
 	}
+	session->timeout = saved_timeout;
 
 	/* IPMI messages are deleted under ipmi_lan_poll_recv() */
 	switch (payload->payload_type) {
@@ -2313,6 +2326,7 @@ ipmi_lanplus_send_payload(
 	case IPMI_PAYLOAD_TYPE_RAKP_1:
 	case IPMI_PAYLOAD_TYPE_RAKP_3:
 		free(msg_data);
+		msg_data = NULL;
 		break;
 	}
 
@@ -2794,6 +2808,7 @@ ipmi_lanplus_open_session(struct ipmi_intf * intf)
 		lprintf(LOG_WARNING, "Unsupported cipher suite ID : %d\n",
 				session->cipher_suite_id);
 		free(msg);
+		msg = NULL;
 		return -1;
 	}
 
@@ -2842,6 +2857,7 @@ ipmi_lanplus_open_session(struct ipmi_intf * intf)
 	rsp = ipmi_lanplus_send_payload(intf, &v2_payload);
 
 	free(msg);
+	msg = NULL;
 
 	if (verbose)
 		lanplus_dump_open_session_response(rsp);
@@ -2972,6 +2988,7 @@ ipmi_lanplus_rakp1(struct ipmi_intf * intf)
 		lprintf(LOG_ERR, "ERROR generating random number "
 			"in ipmi_lanplus_rakp1");
 		free(msg);
+		msg = NULL;
 		return 1;
 	}
 	memcpy(msg + 8, session->v2_data.console_rand, 16);
@@ -3001,6 +3018,7 @@ ipmi_lanplus_rakp1(struct ipmi_intf * intf)
 			"(Exceeds %d characters)",
 			IPMI_MAX_USER_NAME_LENGTH);
 		free(msg);
+		msg = NULL;
 		return 1;
 	}
 	memcpy(msg + 28, session->username, msg[27]);
@@ -3013,6 +3031,7 @@ ipmi_lanplus_rakp1(struct ipmi_intf * intf)
 	rsp = ipmi_lanplus_send_payload(intf, &v2_payload);
 
 	free(msg);
+	msg = NULL;
 
 	if (rsp == NULL)
 	{
@@ -3136,6 +3155,7 @@ ipmi_lanplus_rakp3(struct ipmi_intf * intf)
 			/* Error */
 			lprintf(LOG_INFO, "> Error generating RAKP 3 authcode");
 			free(msg);
+			msg = NULL;
 			return 1;
 		}
 		else
@@ -3145,11 +3165,12 @@ ipmi_lanplus_rakp3(struct ipmi_intf * intf)
 		}
 
 		/* Generate our Session Integrity Key, K1, and K2 */
-		if (lanplus_generate_sik(session))
+		if (lanplus_generate_sik(session, intf))
 		{
 			/* Error */
 			lprintf(LOG_INFO, "> Error generating session integrity key");
 			free(msg);
+			msg = NULL;
 			return 1;
 		}
 		else if (lanplus_generate_k1(session))
@@ -3157,6 +3178,7 @@ ipmi_lanplus_rakp3(struct ipmi_intf * intf)
 			/* Error */
 			lprintf(LOG_INFO, "> Error generating K1 key");
 			free(msg);
+			msg = NULL;
 			return 1;
 		}
 		else if (lanplus_generate_k2(session))
@@ -3164,6 +3186,7 @@ ipmi_lanplus_rakp3(struct ipmi_intf * intf)
 			/* Error */
 			lprintf(LOG_INFO, "> Error generating K1 key");
 			free(msg);
+			msg = NULL;
 			return 1;
 		}
 	}
@@ -3172,6 +3195,7 @@ ipmi_lanplus_rakp3(struct ipmi_intf * intf)
 	rsp = ipmi_lanplus_send_payload(intf, &v2_payload);
 
 	free(msg);
+	msg = NULL;
 
 	if (session->v2_data.rakp2_return_code != IPMI_RAKP_STATUS_NO_ERRORS)
 	{
@@ -3242,8 +3266,10 @@ ipmi_lanplus_close(struct ipmi_intf * intf)
 
 	ipmi_req_clear_entries();
 
-	if (intf->session)
+	if (intf->session) {
 		free(intf->session);
+		intf->session = NULL;
+	}
 
 	intf->session = NULL;
 	intf->opened = 0;
@@ -3278,6 +3304,7 @@ ipmi_set_session_privlvl_cmd(struct ipmi_intf * intf)
 	if (rsp == NULL) {
 		lprintf(LOG_ERR, "Set Session Privilege Level to %s failed",
 			val2str(privlvl, ipmi_privlvl_vals));
+		bridgePossible = backupBridgePossible;
 		return -1;
 	}
 	if (verbose > 2)
@@ -3287,6 +3314,7 @@ ipmi_set_session_privlvl_cmd(struct ipmi_intf * intf)
 		lprintf(LOG_ERR, "Set Session Privilege Level to %s failed: %s",
 			val2str(privlvl, ipmi_privlvl_vals),
 			val2str(rsp->ccode, completion_code_vals));
+		bridgePossible = backupBridgePossible;
 		return -1;
 	}
 
@@ -3359,6 +3387,13 @@ ipmi_lanplus_open(struct ipmi_intf * intf)
 				session->hostname);
 			return -1;
 		}
+		if (host->h_addrtype != AF_INET) {
+			lprintf(LOG_ERR,
+					"Address lookup for %s failed. Got %s, expected IPv4 address.",
+					session->hostname,
+					(host->h_addrtype == AF_INET6) ? "IPv6" : "Unknown");
+			return (-1);
+		}
 		addr.sin_family = host->h_addrtype;
 		memcpy(&addr.sin_addr, host->h_addr, host->h_length);
 	}
@@ -3387,20 +3422,15 @@ ipmi_lanplus_open(struct ipmi_intf * intf)
 	/*
 	 *
 	 * Make sure the BMC supports IPMI v2 / RMCP+
-	 *
-	 * I'm not sure why we accept a failure for the first call
 	 */
-	if (ipmi_get_auth_capabilities_cmd(intf, &auth_cap)) {
-		sleep(1);
-		if (ipmi_get_auth_capabilities_cmd(intf, &auth_cap));
-		{
-			lprintf(LOG_INFO, "Error issuing Get Channel "
-				"Authentication Capabilies request");
-			goto fail;
-		}
+	if (!ipmi_oem_active(intf, "i82571spt") &&
+			ipmi_get_auth_capabilities_cmd(intf, &auth_cap)) {
+		lprintf(LOG_INFO, "Error issuing Get Channel "
+			"Authentication Capabilies request");
+		goto fail;
 	}
 
-	if (! auth_cap.v20_data_available)
+	if (!ipmi_oem_active(intf, "i82571spt") && ! auth_cap.v20_data_available)
 	{
 		lprintf(LOG_INFO, "This BMC does not support IPMI v2 / RMCP+");
 		goto fail;
@@ -3435,13 +3465,14 @@ ipmi_lanplus_open(struct ipmi_intf * intf)
 
 	lprintf(LOG_DEBUG, "IPMIv2 / RMCP+ SESSION OPENED SUCCESSFULLY\n");
 
+	if (!ipmi_oem_active(intf, "i82571spt")) {
+		rc = ipmi_set_session_privlvl_cmd(intf);
+		if (rc < 0)
+			goto fail;
+	}
+	intf->manufacturer_id = ipmi_get_oem(intf);
 	bridgePossible = 1;
 
-	rc = ipmi_set_session_privlvl_cmd(intf);
-	if (rc < 0)
-		goto fail;
-
-	intf->manufacturer_id = ipmi_get_oem(intf);
 	return intf->fd;
 
  fail:
